# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import re
import traceback

from oslo_log import log
from tempest import config
from tempest.lib.common import cred_client
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions
from tempest import test

from manila_tempest_tests import clients
from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests import utils


CONF = config.CONF
LOG = log.getLogger(__name__)

# Test tags related to test direction
TAG_POSITIVE = "positive"
TAG_NEGATIVE = "negative"

# Test tags related to service involvement
# Only requires that manila-api service running.
TAG_API = "api"
# Requires all manila services running, intended to test back-end
# (manila-share) behavior.
TAG_BACKEND = "backend"
# Requires all manila services running, intended to test API behavior.
TAG_API_WITH_BACKEND = "api_with_backend"

TAGS_MAPPER = {
    "p": TAG_POSITIVE,
    "n": TAG_NEGATIVE,
    "a": TAG_API,
    "b": TAG_BACKEND,
    "ab": TAG_API_WITH_BACKEND,
}
TAGS_PATTERN = re.compile(
    r"(?=.*\[.*\b(%(p)s|%(n)s)\b.*\])(?=.*\[.*\b(%(a)s|%(b)s|%(ab)s)\b.*\])" %
    TAGS_MAPPER)

LATEST_MICROVERSION = CONF.share.max_api_microversion


def verify_test_has_appropriate_tags(self):
    if not TAGS_PATTERN.match(self.id()):
        msg = (
            "Required attributes either not set or set improperly. "
            "Two test attributes are expected:\n"
            " - one of '%(p)s' or '%(n)s' and \n"
            " - one of '%(a)s', '%(b)s' or '%(ab)s'."
        ) % TAGS_MAPPER
        raise self.failureException(msg)


class handle_cleanup_exceptions(object):
    """Handle exceptions raised with cleanup operations.

    Always suppress errors when exceptions.NotFound or exceptions.Forbidden
    are raised.
    Suppress all other exceptions only in case config opt
    'suppress_errors_in_cleanup' in config group 'share' is True.
    """

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        if not (isinstance(exc_value,
                           (exceptions.NotFound, exceptions.Forbidden)) or
                CONF.share.suppress_errors_in_cleanup):
            return False  # Do not suppress error if any
        if exc_traceback:
            LOG.error("Suppressed cleanup error in Manila: "
                      "\n%s", traceback.format_exc())
        return True  # Suppress error if any


class BaseSharesTest(test.BaseTestCase):
    """Base test case class for all Manila API tests."""

    credentials = ('primary', )
    force_tenant_isolation = False
    protocols = ["nfs", "cifs", "glusterfs", "hdfs", "cephfs", "maprfs"]

    # Will be cleaned up in resource_cleanup
    class_resources = []

    # Will be cleaned up in tearDown method
    method_resources = []

    # NOTE(andreaf) Override the client manager class to be used, so that
    # a stable class is used, which includes plugin registered services as well
    client_manager = clients.Clients

    @classmethod
    def skip_checks(cls):
        super(BaseSharesTest, cls).skip_checks()
        if not CONF.service_available.manila:
            raise cls.skipException("Manila support is required")
        if not any(p in CONF.share.enable_protocols for p in cls.protocols):
            skip_msg = "%s tests are disabled" % CONF.share.enable_protocols
            raise cls.skipException(skip_msg)

    @classmethod
    def verify_nonempty(cls, *args):
        if not all(args):
            msg = "Missing API credentials in configuration."
            raise cls.skipException(msg)

    @classmethod
    def setup_credentials(cls):
        # This call is used to tell the credential allocator to create
        # network resources for this test case. NOTE: it must go before the
        # super call, to override decisions in the base classes.
        network_resources = {}
        if (CONF.share.multitenancy_enabled and
                CONF.share.create_networks_when_multitenancy_enabled):
            # We're testing a DHSS=True driver, and manila is configured with
            # NeutronNetworkPlugin (or a derivative) that supports creating
            # share networks with project neutron networks, so lets ask for
            # neutron network resources to be created with test credentials
            network_resources.update({'network': True,
                                      'subnet': True,
                                      'router': True})
        cls.set_network_resources(**network_resources)
        super(BaseSharesTest, cls).setup_credentials()

    @classmethod
    def setup_clients(cls):
        super(BaseSharesTest, cls).setup_clients()
        os = getattr(cls, 'os_%s' % cls.credentials[0])
        # Initialise share clients for test credentials
        cls.shares_client = os.share_v1.SharesClient()
        cls.shares_v2_client = os.share_v2.SharesV2Client()
        # Initialise network clients for test credentials
        cls.networks_client = None
        cls.subnets_client = None
        if CONF.service_available.neutron:
            cls.networks_client = os.network.NetworksClient()
            cls.subnets_client = os.network.SubnetsClient()

        # If DHSS=True, create a share network and set it in the client
        # for easy access.
        if CONF.share.multitenancy_enabled:
            if (not CONF.service_available.neutron and
                    CONF.share.create_networks_when_multitenancy_enabled):
                raise cls.skipException(
                    "Neutron support is required when "
                    "CONF.share.create_networks_when_multitenancy_enabled "
                    "is set to True")
            share_network_id = cls.provide_share_network(
                cls.shares_client, cls.networks_client)
            cls.shares_client.share_network_id = share_network_id
            cls.shares_v2_client.share_network_id = share_network_id

    def setUp(self):
        super(BaseSharesTest, self).setUp()
        self.addCleanup(self.clear_resources)
        verify_test_has_appropriate_tags(self)

    @classmethod
    def resource_cleanup(cls):
        cls.clear_resources(cls.class_resources)
        super(BaseSharesTest, cls).resource_cleanup()

    @classmethod
    def provide_and_associate_security_services(
            cls, shares_client, share_network_id, cleanup_in_class=True):
        """Creates a security service and associates to a share network.

        This method creates security services based on the Multiopt
        defined in tempest configuration named security_service. When this
        configuration is not provided, the method will return None.
        After the security service creation, this method also associates
        the security service to a share network.

        :param shares_client: shares client, which requires the provisioning
        :param share_network_id: id of the share network to associate the
            security service
        :param cleanup_in_class: if the security service and the association
            will be removed in the method teardown or class teardown
        :returns: None -- if the security service configuration is not
            defined
        """

        ss_configs = CONF.share.security_service
        if not ss_configs:
            return

        for ss_config in ss_configs:
            ss_name = "ss_autogenerated_by_tempest_%s" % (
                ss_config.get("ss_type"))

            ss_params = {
                "name": ss_name,
                "dns_ip": ss_config.get("ss_dns_ip"),
                "server": ss_config.get("ss_server"),
                "domain": ss_config.get("ss_domain"),
                "user": ss_config.get("ss_user"),
                "password": ss_config.get("ss_password")
            }
            ss_type = ss_config.get("ss_type")
            security_service = cls.create_security_service(
                ss_type,
                client=shares_client,
                cleanup_in_class=cleanup_in_class,
                **ss_params)

            cls.add_sec_service_to_share_network(
                shares_client, share_network_id,
                security_service["id"],
                cleanup_in_class=cleanup_in_class)

    @classmethod
    def add_sec_service_to_share_network(
            cls, client, share_network_id,
            security_service_id, cleanup_in_class=True):
        """Associates a security service to a share network.

        This method associates a security service provided by
        the security service configuration with a specific
        share network.

        :param share_network_id: the share network id to be
            associate with a given security service
        :param security_service_id: the security service id
            to be associate with a given share network
        :param cleanup_in_class: if the resources will be
            dissociate in the method teardown or class teardown
        """

        client.add_sec_service_to_share_network(
            share_network_id,
            security_service_id)
        resource = {
            "type": "dissociate_security_service",
            "id": security_service_id,
            "extra_params": {
                "share_network_id": share_network_id
            },
            "client": client,
        }

        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)

    @classmethod
    def provide_share_network(cls, shares_client, networks_client,
                              ignore_multitenancy_config=False):
        """Get or create share network for DHSS=True drivers

        When testing DHSS=True (multitenancy_enabled) drivers, shares must
        be requested on share networks.
        :returns: str -- share network id for shares_client tenant
        :returns: None -- if single-tenant driver (DHSS=False) is used
        """

        if (not ignore_multitenancy_config and
                not CONF.share.multitenancy_enabled):
            # Assumed usage of a single-tenant driver (DHSS=False)
            return None

        if shares_client.share_network_id:
            # Share-network already exists, use it
            return shares_client.share_network_id

        sn_name = "autogenerated_by_tempest"
        sn_desc = "This share-network was created by tempest"

        if not CONF.share.create_networks_when_multitenancy_enabled:
            # We need a new share network, but don't need to associate
            # any neutron networks to it - this configuration is used
            # when manila is configured with "StandaloneNetworkPlugin"
            # or "NeutronSingleNetworkPlugin" where all tenants share
            # a single backend network where shares are exported.
            sn = cls.create_share_network(cleanup_in_class=True,
                                          client=shares_client,
                                          add_security_services=True,
                                          name=sn_name,
                                          description=sn_desc)
            return sn['id']

        # Retrieve non-public network list owned by the tenant
        filters = {'project_id': shares_client.tenant_id,
                   'shared': False}
        tenant_networks = (
            networks_client.list_networks(**filters).get('networks', [])
        )
        tenant_networks_with_subnet = (
            [n for n in tenant_networks if n['subnets']]
        )

        if not tenant_networks_with_subnet:
            # This can only occur if using tempest's pre-provisioned
            # credentials and not allocating networks to them
            raise cls.skipException(
                "Test credentials must provide at least one "
                "non-shared project network with a valid subnet when "
                "CONF.share.create_networks_when_multitenancy_enabled is "
                "set to True.")

        net_id = tenant_networks_with_subnet[0]['id']
        subnet_id = tenant_networks_with_subnet[0]['subnets'][0]

        # Create suitable share-network
        sn = cls.create_share_network(cleanup_in_class=True,
                                      client=shares_client,
                                      add_security_services=True,
                                      name=sn_name,
                                      description=sn_desc,
                                      neutron_net_id=net_id,
                                      neutron_subnet_id=subnet_id)

        return sn['id']

    @classmethod
    def _create_share(cls, share_protocol=None, size=None, name=None,
                      snapshot_id=None, description=None, metadata=None,
                      share_network_id=None, share_type_id=None,
                      share_group_id=None, client=None,
                      cleanup_in_class=True, is_public=False, **kwargs):
        client = client or cls.shares_v2_client
        description = description or "Tempest's share"
        share_network_id = (share_network_id or
                            CONF.share.share_network_id or
                            client.share_network_id or None)
        metadata = metadata or {}
        size = size or CONF.share.share_size
        kwargs.update({
            'share_protocol': share_protocol,
            'size': size,
            'name': name,
            'snapshot_id': snapshot_id,
            'description': description,
            'metadata': metadata,
            'share_network_id': share_network_id,
            'share_type_id': share_type_id,
            'is_public': is_public,
        })
        if share_group_id:
            kwargs['share_group_id'] = share_group_id

        share = client.create_share(**kwargs)['share']
        resource = {"type": "share", "id": share["id"], "client": client,
                    "share_group_id": share_group_id}
        cleanup_list = (cls.class_resources if cleanup_in_class else
                        cls.method_resources)
        cleanup_list.insert(0, resource)
        return share

    @classmethod
    def migrate_share(
            cls, share_id, dest_host, wait_for_status, client=None,
            force_host_assisted_migration=False, writable=False,
            nondisruptive=False, preserve_metadata=False,
            preserve_snapshots=False, new_share_network_id=None,
            new_share_type_id=None, **kwargs):
        client = client or cls.shares_v2_client
        client.migrate_share(
            share_id, dest_host,
            force_host_assisted_migration=force_host_assisted_migration,
            writable=writable, preserve_metadata=preserve_metadata,
            nondisruptive=nondisruptive, preserve_snapshots=preserve_snapshots,
            new_share_network_id=new_share_network_id,
            new_share_type_id=new_share_type_id, **kwargs)
        share = waiters.wait_for_migration_status(
            client, share_id, dest_host, wait_for_status, **kwargs)
        return share

    @classmethod
    def migration_complete(cls, share_id, dest_host, client=None, **kwargs):
        client = client or cls.shares_v2_client
        client.migration_complete(share_id, **kwargs)
        share = waiters.wait_for_migration_status(
            client, share_id, dest_host, 'migration_success', **kwargs)
        return share

    @classmethod
    def migration_cancel(cls, share_id, dest_host, client=None, **kwargs):
        client = client or cls.shares_v2_client
        client.migration_cancel(share_id, **kwargs)
        share = waiters.wait_for_migration_status(
            client, share_id, dest_host, 'migration_cancelled', **kwargs)
        return share

    @classmethod
    def create_share(cls, *args, **kwargs):
        """Create one share and wait for available state. Retry if allowed."""
        result = cls.create_shares([{"args": args, "kwargs": kwargs}])
        return result[0]

    @classmethod
    def create_shares(cls, share_data_list):
        """Creates several shares in parallel with retries.

        Use this method when you want to create more than one share at same
        time. Especially if config option 'share.share_creation_retry_number'
        has value more than zero (0).
        All shares will be expected to have 'available' status with or without
        recreation else error will be raised.

        :param share_data_list: list -- list of dictionaries with 'args' and
            'kwargs' for '_create_share' method of this base class.
            example of data:
                share_data_list=[{'args': ['quuz'], 'kwargs': {'foo': 'bar'}}}]
        :returns: list -- list of shares created using provided data.
        """

        for d in share_data_list:
            if not isinstance(d, dict):
                raise exceptions.TempestException(
                    "Expected 'dict', got '%s'" % type(d))
            if "args" not in d:
                d["args"] = []
            if "kwargs" not in d:
                d["kwargs"] = {}
            if len(d) > 2:
                raise exceptions.TempestException(
                    "Expected only 'args' and 'kwargs' keys. "
                    "Provided %s" % list(d))

        data = []
        for d in share_data_list:
            client = d["kwargs"].pop("client", cls.shares_v2_client)
            wait_for_status = d["kwargs"].pop("wait_for_status", True)
            local_d = {
                "args": d["args"],
                "kwargs": copy.deepcopy(d["kwargs"]),
            }
            local_d["kwargs"]["client"] = client
            local_d["share"] = cls._create_share(
                *local_d["args"], **local_d["kwargs"])
            local_d["cnt"] = 0
            local_d["available"] = False
            local_d["wait_for_status"] = wait_for_status
            data.append(local_d)

        while not all(d["available"] for d in data):
            for d in data:
                if not d["wait_for_status"]:
                    d["available"] = True
                if d["available"]:
                    continue
                client = d["kwargs"]["client"]
                share_id = d["share"]["id"]
                try:
                    waiters.wait_for_resource_status(
                        client, share_id, "available")
                    d["available"] = True
                except (share_exceptions.ShareBuildErrorException,
                        exceptions.TimeoutException) as e:
                    if CONF.share.share_creation_retry_number > d["cnt"]:
                        d["cnt"] += 1
                        msg = ("Share '%s' failed to be built. "
                               "Trying create another." % share_id)
                        LOG.error(msg)
                        LOG.error(e)
                        cg_id = d["kwargs"].get("consistency_group_id")
                        if cg_id:
                            # NOTE(vponomaryov): delete errored share
                            # immediately in case share is part of CG.
                            client.delete_share(
                                share_id,
                                params={"consistency_group_id": cg_id})
                            client.wait_for_resource_deletion(
                                share_id=share_id)
                        d["share"] = cls._create_share(
                            *d["args"], **d["kwargs"])
                    else:
                        raise

        return [d["share"] for d in data]

    @classmethod
    def create_share_group(cls, client=None, cleanup_in_class=True,
                           share_network_id=None, **kwargs):
        client = client or cls.shares_v2_client
        if kwargs.get('source_share_group_snapshot_id') is None:
            kwargs['share_network_id'] = (share_network_id or
                                          client.share_network_id or None)
        share_group = client.create_share_group(**kwargs)['share_group']
        resource = {
            "type": "share_group",
            "id": share_group["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)

        if kwargs.get('source_share_group_snapshot_id'):
            new_share_group_shares = client.list_shares(
                detailed=True,
                params={'share_group_id': share_group['id']})['shares']

            for share in new_share_group_shares:
                resource = {"type": "share",
                            "id": share["id"],
                            "client": client,
                            "share_group_id": share.get("share_group_id")}
                if cleanup_in_class:
                    cls.class_resources.insert(0, resource)
                else:
                    cls.method_resources.insert(0, resource)

        waiters.wait_for_resource_status(
            client, share_group['id'], 'available',
            resource_name='share_group')
        return share_group

    @classmethod
    def create_share_group_type(cls, name=None, share_types=(), is_public=None,
                                group_specs=None, client=None,
                                cleanup_in_class=True, **kwargs):
        client = client or cls.shares_v2_client
        if (group_specs is None and
                CONF.share.capability_sg_consistent_snapshot_support):
            group_specs = {
                'consistent_snapshot_support': (
                    CONF.share.capability_sg_consistent_snapshot_support),
            }
        share_group_type = client.create_share_group_type(
            name=name,
            share_types=share_types,
            is_public=is_public,
            group_specs=group_specs,
            **kwargs)['share_group_type']
        resource = {
            "type": "share_group_type",
            "id": share_group_type["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_group_type

    @classmethod
    def create_snapshot_wait_for_active(cls, share_id, name=None,
                                        description=None, force=False,
                                        metadata=None, client=None,
                                        cleanup_in_class=True):
        if client is None:
            client = cls.shares_v2_client
        if description is None:
            description = "Tempest's snapshot"
        snapshot = client.create_snapshot(
            share_id, name, description, force, metadata)['snapshot']
        resource = {
            "type": "snapshot",
            "id": snapshot["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        waiters.wait_for_resource_status(client, snapshot["id"], "available",
                                         resource_name='snapshot')
        return snapshot

    @classmethod
    def create_share_group_snapshot_wait_for_active(
            cls, share_group_id, name=None, description=None, client=None,
            cleanup_in_class=True, **kwargs):
        client = client or cls.shares_v2_client
        if description is None:
            description = "Tempest's share group snapshot"
        sg_snapshot = client.create_share_group_snapshot(
            share_group_id, name=name, description=description,
            **kwargs)['share_group_snapshot']
        resource = {
            "type": "share_group_snapshot",
            "id": sg_snapshot["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        waiters.wait_for_resource_status(
            client, sg_snapshot["id"], "available",
            resource_name="share_group_snapshot")
        return sg_snapshot

    @classmethod
    def get_availability_zones(cls, client=None, backends=None):
        """List the availability zones for "manila-share" services

         that are currently in "up" state.
         """
        client = client or cls.admin_shares_v2_client
        backends = (
            '|'.join(['^%s$' % backend for backend in backends])
            if backends else '.*'
        )
        cls.services = client.list_services()['services']
        zones = [service['zone'] for service in cls.services if
                 service['binary'] == 'manila-share' and
                 service['state'] == 'up' and
                 re.search(backends, service['host'])]
        return list(set(zones))

    @classmethod
    def get_pools_matching_share_type(cls, share_type, client=None):
        client = client or cls.admin_shares_v2_client
        if utils.is_microversion_supported('2.23'):
            return client.list_pools(
                detail=True,
                search_opts={'share_type': share_type['id']})['pools']

        pools = client.list_pools(detail=True)['pools']
        share_type = client.get_share_type(share_type['id'])['share_type']
        extra_specs = {}
        for k, v in share_type['extra_specs'].items():
            extra_specs[k] = (
                True if str(v).lower() == 'true'
                else False if str(v).lower() == 'false' else v
            )
        return [
            pool for pool in pools if all(y in pool['capabilities'].items()
                                          for y in extra_specs.items())
        ]

    @classmethod
    def get_availability_zones_matching_share_type(cls, share_type,
                                                   client=None):

        client = client or cls.admin_shares_v2_client
        pools_matching_share_type = cls.get_pools_matching_share_type(
            share_type, client=client)
        backends_matching_share_type = set(
            [pool['name'].split("#")[0] for pool in pools_matching_share_type]
        )
        azs = cls.get_availability_zones(backends=backends_matching_share_type)
        return azs

    def get_pools_for_replication_domain(self, share=None):
        # Get the list of pools for the replication domain
        pools = self.admin_client.list_pools(detail=True)['pools']
        if share:
            instance_host = self.admin_client.get_share(
                share['id'])['share']['host']
        else:
            instance_host = self.admin_client.get_share(
                self.shares[0]['id'])['share']['host']
        host_pool = [p for p in pools if p['name'] == instance_host][0]
        rep_domain = host_pool['capabilities']['replication_domain']
        pools_in_rep_domain = [p for p in pools if p['capabilities'][
            'replication_domain'] == rep_domain]
        return rep_domain, pools_in_rep_domain

    @classmethod
    def create_share_replica(cls, share_id, availability_zone=None,
                             scheduler_hints=None,
                             share_network_id=None,
                             client=None, cleanup_in_class=False,
                             cleanup=True,
                             version=CONF.share.max_api_microversion):
        client = client or cls.shares_v2_client
        replica = client.create_share_replica(
            share_id, availability_zone=availability_zone,
            scheduler_hints=scheduler_hints,
            share_network_id=share_network_id,
            version=version)['share_replica']
        resource = {
            "type": "share_replica",
            "id": replica["id"],
            "client": client,
            "share_id": share_id,
        }
        # NOTE(Yogi1): Cleanup needs to be disabled during promotion tests.
        if cleanup:
            if cleanup_in_class:
                cls.class_resources.insert(0, resource)
            else:
                cls.method_resources.insert(0, resource)
        waiters.wait_for_resource_status(
            client, replica["id"], constants.STATUS_AVAILABLE,
            resource_name='share_replica')
        return replica

    @classmethod
    def create_backup_wait_for_active(cls, share_id, client=None,
                                      cleanup_in_class=False, cleanup=True,
                                      version=CONF.share.max_api_microversion):
        client = client or cls.shares_v2_client
        backup_name = data_utils.rand_name('Backup')
        backup_options = CONF.share.driver_assisted_backup_test_driver_options
        backup = client.create_share_backup(
            share_id,
            name=backup_name,
            backup_options=backup_options)['share_backup']
        resource = {
            "type": "share_backup",
            "id": backup["id"],
            "client": client,
        }
        if cleanup:
            if cleanup_in_class:
                cls.class_resources.insert(0, resource)
            else:
                cls.method_resources.insert(0, resource)
        waiters.wait_for_resource_status(client, backup["id"], "available",
                                         resource_name='share_backup')
        return client.get_share_backup(backup['id'])['share_backup']

    @classmethod
    def delete_share_replica(cls, replica_id, client=None,
                             version=CONF.share.max_api_microversion):
        client = client or cls.shares_v2_client
        try:
            client.delete_share_replica(replica_id, version=version)
            client.wait_for_resource_deletion(replica_id=replica_id)
        except exceptions.NotFound:
            pass

    @classmethod
    def promote_share_replica(cls, replica_id, client=None,
                              version=CONF.share.max_api_microversion):
        client = client or cls.shares_v2_client
        replica = client.promote_share_replica(
            replica_id, version=version)['share_replica']
        waiters.wait_for_resource_status(
            client, replica["id"], constants.REPLICATION_STATE_ACTIVE,
            resource_name='share_replica', status_attr="replica_state")
        return replica

    @classmethod
    def create_share_network(cls, client=None,
                             cleanup_in_class=False,
                             add_security_services=True, **kwargs):

        if client is None:
            client = cls.shares_client
        share_network = client.create_share_network(**kwargs)['share_network']
        resource = {
            "type": "share_network",
            "id": share_network["id"],
            "client": client,
        }

        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)

        if add_security_services:
            cls.provide_and_associate_security_services(
                client, share_network["id"], cleanup_in_class=cleanup_in_class)

        return share_network

    @classmethod
    def create_share_network_subnet(cls,
                                    client=None,
                                    cleanup_in_class=False,
                                    **kwargs):
        if client is None:
            client = cls.shares_v2_client
        share_network_subnet = client.create_subnet(
            **kwargs)['share_network_subnet']
        resource = {
            "type": "share_network_subnet",
            "id": share_network_subnet["id"],
            "extra_params": {
                "share_network_id": share_network_subnet["share_network_id"]
            },
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_network_subnet

    @classmethod
    def create_security_service(cls, ss_type="ldap", client=None,
                                cleanup_in_class=False, **kwargs):
        if client is None:
            client = cls.shares_client
        security_service = client.create_security_service(
            ss_type, **kwargs)['security_service']
        resource = {
            "type": "security_service",
            "id": security_service["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return security_service

    @classmethod
    def create_resource_lock(cls, resource_id, resource_type='share',
                             resource_action='delete', lock_reason=None,
                             client=None, version=LATEST_MICROVERSION,
                             cleanup_in_class=True):
        lock_reason = lock_reason or "locked by tempest tests"
        client = client or cls.shares_v2_client

        lock = client.create_resource_lock(resource_id,
                                           resource_type,
                                           resource_action=resource_action,
                                           lock_reason=lock_reason,
                                           version=version)['resource_lock']
        resource = {
            "type": "resource_lock",
            "id": lock["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return lock

    @classmethod
    def update_share_type(cls, share_type_id, name=None,
                          is_public=None, description=None,
                          client=None):
        if client is None:
            client = cls.shares_v2_client
        share_type = client.update_share_type(share_type_id, name,
                                              is_public, description)
        return share_type

    @classmethod
    def update_quotas(cls, project_id, user_id=None, cleanup=True,
                      client=None, **kwargs):
        client = client or cls.shares_v2_client
        updated_quotas = client.update_quotas(project_id,
                                              user_id=user_id,
                                              **kwargs)['quota_set']
        resource = {
            "type": "quotas",
            "id": project_id,
            "client": client,
            "user_id": user_id,
        }
        if cleanup:
            cls.method_resources.insert(0, resource)
        return updated_quotas

    @classmethod
    def clear_share_replicas(cls, share_id, client=None):
        client = client or cls.shares_v2_client
        share_replicas = client.list_share_replicas(
            share_id=share_id)['share_replicas']

        for replica in share_replicas:
            try:
                cls.delete_share_replica(replica['id'])
            except exceptions.BadRequest:
                # Ignore the exception due to deletion of last active replica
                pass

    @classmethod
    def clear_resources(cls, resources=None):
        """Deletes resources, that were created in test suites.

        This method tries to remove resources from resource list,
        if it is not found, assumed it was deleted in test itself.
        It is expected, that all resources were added as LIFO
        due to restriction of deletion resources, that is in the chain.

        :param resources: dict with keys 'type','id','client' and 'deleted'
        """
        if resources is None:
            resources = cls.method_resources
        for res in resources:
            if "deleted" not in res.keys():
                res["deleted"] = False
            if "client" not in res.keys():
                res["client"] = cls.shares_client
            if not(res["deleted"]):
                res_id = res['id']
                client = res["client"]
                with handle_cleanup_exceptions():
                    if res["type"] == "share":
                        cls.clear_share_replicas(res_id)
                        share_group_id = res.get('share_group_id')
                        if share_group_id:
                            params = {'share_group_id': share_group_id}
                            client.delete_share(res_id, params=params)
                        else:
                            client.delete_share(res_id)
                        client.wait_for_resource_deletion(share_id=res_id)
                    elif res["type"] == "snapshot":
                        client.delete_snapshot(res_id)
                        client.wait_for_resource_deletion(snapshot_id=res_id)
                    elif (res["type"] == "share_network" and
                            res_id != CONF.share.share_network_id):
                        client.delete_share_network(res_id)
                        client.wait_for_resource_deletion(sn_id=res_id)
                    elif res["type"] == "dissociate_security_service":
                        sn_id = res["extra_params"]["share_network_id"]
                        client.remove_sec_service_from_share_network(
                            sn_id=sn_id, ss_id=res_id
                        )
                    elif res["type"] == "security_service":
                        client.delete_security_service(res_id)
                        client.wait_for_resource_deletion(ss_id=res_id)
                    elif res["type"] == "share_type":
                        client.delete_share_type(res_id)
                        client.wait_for_resource_deletion(st_id=res_id)
                    elif res["type"] == "share_group":
                        client.delete_share_group(res_id)
                        client.wait_for_resource_deletion(
                            share_group_id=res_id)
                    elif res["type"] == "share_group_type":
                        client.delete_share_group_type(res_id)
                        client.wait_for_resource_deletion(
                            share_group_type_id=res_id)
                    elif res["type"] == "share_group_snapshot":
                        client.delete_share_group_snapshot(res_id)
                        client.wait_for_resource_deletion(
                            share_group_snapshot_id=res_id)
                    elif res["type"] == "share_replica":
                        client.delete_share_replica(res_id)
                        client.wait_for_resource_deletion(replica_id=res_id)
                    elif res["type"] == "share_backup":
                        client.delete_share_backup(res_id)
                        client.wait_for_resource_deletion(backup_id=res_id)
                    elif res["type"] == "share_network_subnet":
                        sn_id = res["extra_params"]["share_network_id"]
                        client.delete_subnet(sn_id, res_id)
                        client.wait_for_resource_deletion(
                            share_network_subnet_id=res_id,
                            sn_id=sn_id)
                    elif res["type"] == "quotas":
                        user_id = res.get('user_id')
                        client.reset_quotas(res_id, user_id=user_id)
                    elif res["type"] == "resource_lock":
                        client.delete_resource_lock(res_id)
                    else:
                        LOG.warning("Provided unsupported resource type for "
                                    "cleanup '%s'. Skipping.", res["type"])
                res["deleted"] = True

    # Useful assertions
    def assertDictMatch(self, d1, d2, approx_equal=False, tolerance=0.001):
        """Assert two dicts are equivalent.

        This is a 'deep' match in the sense that it handles nested
        dictionaries appropriately.

        NOTE:

            If you don't care (or don't know) a given value, you can specify
            the string DONTCARE as the value. This will cause that dict-item
            to be skipped.

        """
        def raise_assertion(msg):
            d1str = str(d1)
            d2str = str(d2)
            base_msg = ('Dictionaries do not match. %(msg)s d1: %(d1str)s '
                        'd2: %(d2str)s' %
                        {"msg": msg, "d1str": d1str, "d2str": d2str})
            raise AssertionError(base_msg)

        d1keys = set(d1.keys())
        d2keys = set(d2.keys())
        if d1keys != d2keys:
            d1only = d1keys - d2keys
            d2only = d2keys - d1keys
            raise_assertion('Keys in d1 and not d2: %(d1only)s. '
                            'Keys in d2 and not d1: %(d2only)s' %
                            {"d1only": d1only, "d2only": d2only})

        for key in d1keys:
            d1value = d1[key]
            d2value = d2[key]
            try:
                error = abs(float(d1value) - float(d2value))
                within_tolerance = error <= tolerance
            except (ValueError, TypeError):
                # If both values aren't convertible to float, just ignore
                # ValueError if arg is a str, TypeError if it's something else
                # (like None)
                within_tolerance = False

            if hasattr(d1value, 'keys') and hasattr(d2value, 'keys'):
                self.assertDictMatch(d1value, d2value)
            elif 'DONTCARE' in (d1value, d2value):
                continue
            elif approx_equal and within_tolerance:
                continue
            elif d1value != d2value:
                raise_assertion("d1['%(key)s']=%(d1value)s != "
                                "d2['%(key)s']=%(d2value)s" %
                                {
                                    "key": key,
                                    "d1value": d1value,
                                    "d2value": d2value
                                })

    def create_user_message(self):
        """Trigger a 'no valid host' situation to generate a message."""
        extra_specs = {
            'vendor_name': 'foobar',
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        share_type_name = data_utils.rand_name("share-type")

        bogus_type = self.create_share_type(
            client=self.admin_shares_v2_client,
            name=share_type_name,
            extra_specs=extra_specs)['share_type']

        params = {'share_type_id': bogus_type['id'],
                  'share_network_id': self.shares_v2_client.share_network_id}
        share = self.shares_v2_client.create_share(**params)['share']
        self.addCleanup(self.shares_v2_client.delete_share, share['id'])
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], "error")
        return waiters.wait_for_message(self.shares_v2_client, share['id'])

    def allow_access(self, share_id, client=None, access_type=None,
                     access_level='rw', access_to=None, metadata=None,
                     version=LATEST_MICROVERSION, status='active',
                     raise_rule_in_error_state=True, lock_visibility=False,
                     lock_deletion=False, cleanup=True):

        client = client or self.shares_v2_client
        a_type, a_to = utils.get_access_rule_data_from_config(
            client.share_protocol)
        access_type = access_type or a_type
        access_to = access_to or a_to

        kwargs = {
            'access_type': access_type,
            'access_to': access_to,
            'access_level': access_level
        }
        delete_kwargs = (
            {'unrestrict': True} if lock_deletion else {}
        )
        if client is self.shares_v2_client:
            kwargs.update({'metadata': metadata, 'version': version})
        if lock_visibility:
            kwargs.update({'lock_visibility': True})
        if lock_deletion:
            kwargs.update({'lock_deletion': True})

        rule = client.create_access_rule(share_id, **kwargs)['access']
        waiters.wait_for_resource_status(
            client, share_id, status, resource_name='access_rule',
            rule_id=rule['id'], version=version,
            raise_rule_in_error_state=raise_rule_in_error_state)
        if cleanup:
            self.addCleanup(
                client.wait_for_resource_deletion, rule_id=rule['id'],
                share_id=share_id, version=version)
            self.addCleanup(
                client.delete_access_rule, share_id, rule['id'],
                **delete_kwargs)
        return rule


class BaseSharesAdminTest(BaseSharesTest):
    """Base test case class for all Shares Admin API tests."""
    credentials = ('admin', )

    @classmethod
    def setup_clients(cls):
        super(BaseSharesAdminTest, cls).setup_clients()
        # Initialise share clients
        cls.admin_shares_client = cls.os_admin.share_v1.SharesClient()
        cls.admin_shares_v2_client = cls.os_admin.share_v2.SharesV2Client()

    @staticmethod
    def add_extra_specs_to_dict(extra_specs=None):
        """Add any required extra-specs to share type dictionary"""
        dhss = str(CONF.share.multitenancy_enabled)
        extra_specs_dict = {"driver_handles_share_servers": dhss}
        if extra_specs:
            extra_specs_dict.update(extra_specs)
        if CONF.share.capability_thin_provisioned:
            extra_specs_dict['thin_provisioning'] = 'True'
        return extra_specs_dict

    @classmethod
    def create_share_type(cls, name=None, is_public=True, client=None,
                          cleanup_in_class=True, extra_specs=None, **kwargs):
        name = name or data_utils.rand_name(
            cls.__class__.__name__ + 'share-type')
        client = client or cls.admin_shares_v2_client
        extra_specs = cls.add_extra_specs_to_dict(extra_specs=extra_specs)
        share_type = client.create_share_type(name, is_public,
                                              extra_specs=extra_specs,
                                              **kwargs)['share_type']
        resource = {
            "type": "share_type",
            "id": share_type["id"],
            "client": client,
        }
        if cleanup_in_class:
            cls.class_resources.insert(0, resource)
        else:
            cls.method_resources.insert(0, resource)
        return share_type

    @classmethod
    def _create_share_group_type(cls):
        share_group_type_name = data_utils.rand_name("unique_sgtype_name")
        return cls.create_share_group_type(
            name=share_group_type_name, share_types=[cls.share_type_id],
            client=cls.admin_shares_v2_client)

    def _create_share_for_manage(self):
        creation_data = {
            'share_type_id': self.st['id'],
            'share_protocol': self.protocol,
        }

        share = self.create_share(**creation_data)
        share = self.shares_v2_client.get_share(share['id'])['share']

        if utils.is_microversion_ge(CONF.share.max_api_microversion, "2.9"):
            el = self.shares_v2_client.list_share_export_locations(
                share["id"])['export_locations']
            share["export_locations"] = el

        return share

    def _unmanage_share_and_wait(self, share):
        self.shares_v2_client.unmanage_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])

    def _reset_state_and_delete_share(self, share):
        self.shares_v2_client.reset_state(share['id'])
        self._delete_share_and_wait(share)

    def _delete_snapshot_and_wait(self, snap):
        self.shares_v2_client.delete_snapshot(snap['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            snapshot_id=snap['id']
        )
        self.assertRaises(exceptions.NotFound,
                          self.shares_v2_client.get_snapshot,
                          snap['id'])

    def _delete_share_and_wait(self, share):
        self.shares_v2_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])
        self.assertRaises(exceptions.NotFound,
                          self.shares_v2_client.get_share,
                          share['id'])

    def _manage_share(self, share, name, description, share_server_id):
        managed_share = self.shares_v2_client.manage_share(
            service_host=share['host'],
            export_path=share['export_locations'][0],
            protocol=share['share_proto'],
            share_type_id=self.share_type['id'],
            name=name,
            description=description,
            share_server_id=share_server_id
        )['share']
        waiters.wait_for_resource_status(
            self.shares_v2_client, managed_share['id'],
            constants.STATUS_AVAILABLE
        )

        return managed_share

    def _unmanage_share_server_and_wait(self, server):
        self.shares_v2_client.unmanage_share_server(server['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            server_id=server['id']
        )

    def _manage_share_server(self, share_server, fields=None):
        params = fields or {}
        subnet_id = params.get('share_network_subnet_id', None)
        managed_share_server = self.shares_v2_client.manage_share_server(
            params.get('host', share_server['host']),
            params.get('share_network_id', share_server['share_network_id']),
            params.get('identifier', share_server['identifier']),
            share_network_subnet_id=subnet_id,
        )['share_server']
        waiters.wait_for_resource_status(
            self.shares_v2_client, managed_share_server['id'],
            constants.SERVER_STATE_ACTIVE, resource_name='share_server'
        )

        return managed_share_server

    def _delete_share_server_and_wait(self, share_server_id):
        self.shares_v2_client.delete_share_server(
            share_server_id
        )
        self.shares_v2_client.wait_for_resource_deletion(
            server_id=share_server_id)

    def create_user_message(self):
        """Trigger a 'no valid host' situation to generate a message."""
        extra_specs = {
            'vendor_name': 'foobar',
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        share_type_name = data_utils.rand_name("share-type")

        bogus_type = self.create_share_type(
            client=self.admin_shares_v2_client,
            name=share_type_name,
            extra_specs=extra_specs)

        params = {'share_type_id': bogus_type['id'],
                  'share_network_id': self.shares_v2_client.share_network_id}
        share = self.shares_v2_client.create_share(**params)['share']
        self.addCleanup(self.shares_v2_client.delete_share, share['id'])
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], "error")
        return waiters.wait_for_message(self.shares_v2_client, share['id'])


class BaseSharesMixedTest(BaseSharesAdminTest):
    """Base test case class for all Shares API tests with all user roles.

       Tests deriving from this class can use the primary project's clients
       (self.shares_client, self.shares_v2_client) and the alt project user's
       clients (self.alt_shares_client, self.alt_shares_v2_client) to perform
       API calls and validations. Although admin clients are available for use,
       their use should be limited to performing bootstrapping (e.g., creating
       a share type, or resetting state of a resource, etc.). No API validation
       must be performed against admin APIs. Use BaseAdminTest as a base class
       for such tests.
    """
    credentials = ('primary', 'alt', 'admin')

    # Will be cleaned up in resource_cleanup if the class
    class_project_users_created = []

    @classmethod
    def resource_cleanup(cls):
        cls.clear_project_users(cls.class_project_users_created)
        super(BaseSharesMixedTest, cls).resource_cleanup()

    @classmethod
    def clear_project_users(cls, users=None):
        users = users or cls.class_project_users_created
        for user in users:
            with handle_cleanup_exceptions():
                cls.os_admin.creds_client.delete_user(user['id'])

    @classmethod
    def setup_clients(cls):
        super(BaseSharesMixedTest, cls).setup_clients()
        cls.alt_shares_client = cls.os_alt.share_v1.SharesClient()
        cls.alt_shares_v2_client = cls.os_alt.share_v2.SharesV2Client()
        # Initialise network clients
        cls.os_admin.networks_client = cls.os_admin.network.NetworksClient()
        cls.os_alt.networks_client = cls.os_alt.network.NetworksClient()
        # Initialise identity clients
        cls.admin_project = cls.os_admin.auth_provider.auth_data[1]['project']
        identity_clients = getattr(
            cls.os_admin, 'identity_%s' % CONF.identity.auth_version)
        cls.os_admin.identity_client = identity_clients.IdentityClient()
        cls.os_admin.projects_client = identity_clients.ProjectsClient()
        cls.os_admin.users_client = identity_clients.UsersClient()
        cls.os_admin.roles_client = identity_clients.RolesClient()
        cls.os_admin.domains_client = (
            cls.os_admin.identity_v3.DomainsClient() if
            CONF.identity.auth_version == 'v3' else None)
        cls.admin_project_member_client = cls.create_user_and_get_client(
            project=cls.admin_project, add_member_role=True)

        if CONF.share.multitenancy_enabled:
            admin_share_network_id = cls.provide_share_network(
                cls.admin_shares_v2_client, cls.os_admin.networks_client)
            cls.admin_shares_client.share_network_id = admin_share_network_id
            cls.admin_shares_v2_client.share_network_id = (
                admin_share_network_id)

            alt_share_network_id = cls.provide_share_network(
                cls.alt_shares_v2_client, cls.os_alt.networks_client)
            cls.alt_shares_client.share_network_id = alt_share_network_id
            cls.alt_shares_v2_client.share_network_id = alt_share_network_id

    @classmethod
    def create_user_and_get_client(cls, project=None, add_member_role=True):
        """Create a user in specified project & set share clients for user

        The user will have all roles specified in tempest.conf
        :param: project: a dictionary with project ID and name, if not
            specified, the value will be cls.admin_project
        """
        project_domain_name = (
            cls.os_admin.identity_client.auth_provider.credentials.get(
                'project_domain_name', 'Default'))
        cls.os_admin.creds_client = cred_client.get_creds_client(
            cls.os_admin.identity_client, cls.os_admin.projects_client,
            cls.os_admin.users_client, cls.os_admin.roles_client,
            cls.os_admin.domains_client, project_domain_name)

        # User info
        project = project or cls.admin_project
        username = data_utils.rand_name('manila_%s' % project['id'])
        password = data_utils.rand_password()
        email = '%s@example.org' % username

        user = cls.os_admin.creds_client.create_user(
            username, password, project, email)
        cls.class_project_users_created.append(user)

        tempest_roles_to_assign = CONF.auth.tempest_roles or []
        if "member" not in tempest_roles_to_assign and add_member_role:
            tempest_roles_to_assign.append("member")

        for role in tempest_roles_to_assign:
            cls.os_admin.creds_client.assign_user_role(user, project, role)

        user_creds = cls.os_admin.creds_client.get_credentials(
            user, project, password)
        os = clients.Clients(user_creds)
        os.shares_v1_client = os.share_v1.SharesClient()
        os.shares_v2_client = os.share_v2.SharesV2Client()
        return os



# Copyright 2017 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareGroupsNegativeTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ShareGroupsNegativeTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('b90537b7-634d-4fca-b451-770fbcca7927')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share_group_with_wrong_consistent_snapshot_spec(self):
        # Create valid share type for share group type
        share_type = self.create_share_type(cleanup_in_class=False)

        # Create share group type with wrong value for
        # 'consistent_snapshot_support' capability, we always expect
        # NoValidHostFound using this SG type.
        sg_type = self.create_share_group_type(
            name=data_utils.rand_name("tempest-manila"),
            share_types=[share_type['id']],
            group_specs={"consistent_snapshot_support": "fake"},
            cleanup_in_class=False)

        # Try create share group
        self.assertRaises(
            share_exceptions.ShareGroupBuildErrorException,
            self.create_share_group,
            share_type_ids=[share_type['id']],
            share_group_type_id=sg_type['id'],
            cleanup_in_class=False)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_utils import timeutils
from oslo_utils import uuidutils
from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ExportLocationsTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ExportLocationsTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported('2.9')

    @classmethod
    def resource_setup(cls):
        super(ExportLocationsTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.admin_project_member_client.shares_v2_client
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id,
                                     client=cls.admin_client)
        cls.share = cls.admin_client.get_share(cls.share['id'])['share']
        cls.share_instances = cls.admin_client.get_instances_of_share(
            cls.share['id'])['share_instances']

    def _verify_export_location_structure(
            self, export_locations, role='admin', version=LATEST_MICROVERSION,
            format='summary'):

        # Determine which keys to expect based on role, version and format
        summary_keys = ['id', 'path']
        if utils.is_microversion_ge(version, '2.14'):
            summary_keys += ['preferred']

        admin_summary_keys = summary_keys + [
            'share_instance_id', 'is_admin_only']

        detail_keys = summary_keys + ['created_at', 'updated_at']

        admin_detail_keys = admin_summary_keys + ['created_at', 'updated_at']

        if format == 'summary':
            if role == 'admin':
                expected_keys = admin_summary_keys
            else:
                expected_keys = summary_keys
        else:
            if role == 'admin':
                expected_keys = admin_detail_keys
            else:
                expected_keys = detail_keys

        if not isinstance(export_locations, (list, tuple, set)):
            export_locations = (export_locations, )

        for export_location in export_locations:

            # Check that the correct keys are present
            self.assertEqual(len(expected_keys), len(export_location))
            for key in expected_keys:
                self.assertIn(key, export_location)

            # Check the format of ever-present summary keys
            self.assertTrue(uuidutils.is_uuid_like(export_location['id']))
            self.assertIsInstance(export_location['path'],
                                  str)

            if utils.is_microversion_ge(version, '2.14'):
                self.assertIn(export_location['preferred'], (True, False))

            if role == 'admin':
                self.assertIn(export_location['is_admin_only'], (True, False))
                self.assertTrue(uuidutils.is_uuid_like(
                    export_location['share_instance_id']))

            # Check the format of the detail keys
            if format == 'detail':
                for time in (export_location['created_at'],
                             export_location['updated_at']):
                    # If var 'time' has incorrect value then ValueError
                    # exception is expected to be raised. So, just try parse
                    # it making assertion that it has proper date value.
                    timeutils.parse_strtime(time)

    @decorators.idempotent_id('dfcb05af-369a-44c9-a06a-67d12a2a0917')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.13')
    def test_list_share_export_locations(self):
        export_locations = self.admin_client.list_share_export_locations(
            self.share['id'], version='2.13')['export_locations']

        self._verify_export_location_structure(export_locations,
                                               version='2.13')

    @decorators.idempotent_id('032173d7-3ddf-4730-8524-d1a96a2a9e16')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.14')
    def test_list_share_export_locations_with_preferred_flag(self):
        export_locations = self.admin_client.list_share_export_locations(
            self.share['id'], version='2.14')['export_locations']

        self._verify_export_location_structure(export_locations,
                                               version='2.14')

    @decorators.idempotent_id('814da2ce-2909-4b02-a92e-12bc1b640580')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_export_location(self):
        export_locations = self.admin_client.list_share_export_locations(
            self.share['id'])['export_locations']

        for export_location in export_locations:
            el = self.admin_client.get_share_export_location(
                self.share['id'], export_location['id'])['export_location']
            self._verify_export_location_structure(el, format='detail')

    @decorators.idempotent_id('397969c6-7fc8-4bf8-86c7-300b96857c54')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_export_locations_by_member(self):
        export_locations = self.member_client.list_share_export_locations(
            self.share['id'])['export_locations']

        self._verify_export_location_structure(export_locations, role='member')

    @decorators.idempotent_id('66cef86f-5da8-4cb4-bc21-91f6c1e27cb5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_export_location_by_member(self):
        export_locations = self.admin_client.list_share_export_locations(
            self.share['id'])['export_locations']

        for export_location in export_locations:
            if export_location['is_admin_only']:
                continue
            el = self.member_client.get_share_export_location(
                self.share['id'], export_location['id'])['export_location']
            self._verify_export_location_structure(el, role='member',
                                                   format='detail')

    @decorators.idempotent_id('06ea2636-1c9f-4889-8b5f-e10c2c2572cb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.13')
    def test_list_share_instance_export_locations(self):
        for share_instance in self.share_instances:
            export_locations = (
                self.admin_client.list_share_instance_export_locations(
                    share_instance['id'], version='2.13'))['export_locations']
            self._verify_export_location_structure(export_locations,
                                                   version='2.13')

    @decorators.idempotent_id('b93e4cba-ea98-4b1c-90f8-e0a8763033a3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.14')
    def test_list_share_instance_export_locations_with_preferred_flag(self):
        for share_instance in self.share_instances:
            export_locations = (
                self.admin_client.list_share_instance_export_locations(
                    share_instance['id'], version='2.14'))['export_locations']
            self._verify_export_location_structure(export_locations,
                                                   version='2.14')

    @decorators.idempotent_id('59421c43-293f-41fd-8ac6-e856deeceac9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_instance_export_location(self):
        for share_instance in self.share_instances:
            export_locations = (
                self.admin_client.list_share_instance_export_locations(
                    share_instance['id'])['export_locations'])
            for el in export_locations:
                el = self.admin_client.get_share_instance_export_location(
                    share_instance['id'], el['id'])['export_location']
                self._verify_export_location_structure(el, format='detail')

    @decorators.idempotent_id('581acd8d-b89d-4684-8310-b910b46acc7a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_share_contains_all_export_locations_of_all_share_instances(self):
        share_export_locations = self.admin_client.list_share_export_locations(
            self.share['id'])['export_locations']
        share_instances_export_locations = []
        for share_instance in self.share_instances:
            share_instance_export_locations = (
                self.admin_client.list_share_instance_export_locations(
                    share_instance['id'])['export_locations'])
            share_instances_export_locations.extend(
                share_instance_export_locations)

        self.assertEqual(
            len(share_export_locations),
            len(share_instances_export_locations)
        )
        self.assertEqual(
            sorted(share_export_locations, key=lambda el: el['id']),
            sorted(share_instances_export_locations, key=lambda el: el['id'])
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

tc = testtools.testcase
CONF = config.CONF


@ddt.ddt
class ShareTypeAvailabilityZonesTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareTypeAvailabilityZonesTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.48")

    @classmethod
    def resource_setup(cls):
        super(ShareTypeAvailabilityZonesTest, cls).resource_setup()
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']
        all_azs = cls.get_availability_zones()
        cls.valid_azs = cls.get_availability_zones_matching_share_type(
            cls.share_type)
        cls.invalid_azs = ((set(all_azs) - set(cls.valid_azs))
                           or ['az_that_doesnt_exist'])
        cls.az_spec = 'availability_zones'
        cls.valid_azs_spec = ', '.join(cls.valid_azs)
        cls.invalid_azs_spec = ', '.join(cls.invalid_azs)

    @decorators.idempotent_id('23005fad-59e1-4a4b-994c-7377e34b4ec6')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('az1,     az2, az 3  ', 'az1,az2,az 3', 'az1   ,az2,   az 3')
    def test_share_type_azs_create_and_get_share_type(self, spec):
        az_spec = {'availability_zones': spec}
        extra_specs = self.add_extra_specs_to_dict(az_spec)
        share_type = self.create_share_type(
            data_utils.rand_name('az_share_type'),
            cleanup_in_class=False,
            extra_specs=extra_specs,
            client=self.admin_shares_v2_client)
        self.assertEqual(
            'az1,az2,az 3', share_type['extra_specs']['availability_zones'])

        share_type = self.admin_shares_v2_client.get_share_type(
            share_type['id'])['share_type']
        self.assertEqual(
            'az1,az2,az 3', share_type['extra_specs']['availability_zones'])

    @decorators.idempotent_id('545f1742-39dc-4bfb-8a81-6ec941b58fef')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('az1', 'az2', 'az 3', 'az1,  az 3', 'az 3, az1',
              'az2, az 3, az1')
    def test_share_type_azs_filter_by_availability_zones(self, filter):
        az_spec = {'availability_zones': 'az1, az2, az 3'}
        extra_specs = self.add_extra_specs_to_dict(az_spec)
        share_type_in_specific_azs = self.create_share_type(
            data_utils.rand_name('support_some_azs_share_type'),
            cleanup_in_class=False,
            extra_specs=extra_specs,
            client=self.admin_shares_v2_client)

        extra_specs = self.add_extra_specs_to_dict()
        share_type_no_az_spec = self.create_share_type(
            data_utils.rand_name('support_any_az_share_type'),
            cleanup_in_class=False,
            extra_specs=extra_specs,
            client=self.admin_shares_v2_client)

        share_types = self.admin_shares_v2_client.list_share_types(
            params={'extra_specs': {'availability_zones': filter}}
        )['share_types']
        share_type_ids = [s['id'] for s in share_types]
        self.assertIn(share_type_in_specific_azs['id'], share_type_ids)
        self.assertIn(share_type_no_az_spec['id'], share_type_ids)

    @decorators.idempotent_id('00a3e4ab-00a5-4683-a773-166353a69f28')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_share_type_azs_old_version_api_ignores_spec(self):
        """< v2.48, configuring share type AZs shouldn't fail share creation"""
        # Use valid AZs as share_type's availability_zones
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.valid_azs_spec)
        self.create_share(share_type_id=self.share_type_id,
                          cleanup_in_class=False, version='2.47')

        # Use invalid AZs as share_type's availability_zones
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.invalid_azs_spec)
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False, version='2.47')
        share = self.shares_v2_client.get_share(share['id'])['share']
        # Test default scheduler behavior: the share type capabilities should
        # have ensured the share landed in an AZ that is supported
        # regardless of the 'availability_zones' extra-spec
        self.assertIn(share['availability_zone'], self.valid_azs)

    @decorators.idempotent_id('3399c706-5dcb-41b2-9701-4550bf4f81cb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(True, False)
    def test_share_type_azs_shares_az_in_create_req(self, specify_az):
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.valid_azs_spec)
        kwargs = {
            'share_type_id': self.share_type_id,
            'cleanup_in_class': False,
            'availability_zone': self.valid_azs[0] if specify_az else None,
        }
        share = self.create_share(**kwargs)
        share = self.shares_v2_client.get_share(share['id'])['share']
        if specify_az:
            self.assertEqual(self.valid_azs[0], share['availability_zone'])
        else:
            self.assertIn(share['availability_zone'], self.valid_azs)

    @decorators.idempotent_id('fa5618ad-c452-41bd-8dc3-cc85abd6cee5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(True, False)
    def test_share_type_azs_share_groups_az_in_create_req(self, specify_az):
        self.admin_shares_v2_client.update_share_type_extra_spec(
            self.share_type_id, self.az_spec, self.valid_azs_spec)
        kwargs = {
            'share_group_type_id': self.share_group_type_id,
            'share_type_ids': [self.share_type_id],
            'cleanup_in_class': False,
            'availability_zone': self.valid_azs[0] if specify_az else None,
        }
        # Create share group
        share_group = self.create_share_group(**kwargs)
        share_group = self.shares_v2_client.get_share_group(
            share_group['id'])['share_group']
        if specify_az:
            self.assertEqual(self.valid_azs[0],
                             share_group['availability_zone'])
        else:
            self.assertIn(share_group['availability_zone'], self.valid_azs)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


class PublicSharesNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(PublicSharesNegativeTest, cls).resource_setup()
        # create share_type
        share_type = cls.create_share_type()
        share_type_id = share_type['id']
        # create a public share - manila's default RBAC only allows
        # administrator users operating at system scope to create public shares
        cls.share = cls.create_share(
            name='public_share',
            description='public_share_desc',
            share_type_id=share_type_id,
            is_public=True,
            metadata={'key': 'value'},
            client=cls.admin_shares_v2_client
        )

    @decorators.idempotent_id('255011c0-4ed9-4174-bb13-8bbd06a62529')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_share_with_wrong_public_value(self):
        self.assertRaises(lib_exc.BadRequest,
                          self.admin_shares_v2_client.update_share,
                          self.share["id"],
                          is_public="truebar")

    @decorators.idempotent_id('3443493b-f56a-4faa-9968-e7cbb0d2802f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_other_tenants_public_share(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_shares_v2_client.update_share,
                          self.share["id"],
                          name="new_name")

    @decorators.idempotent_id('68d1f1bc-16e4-4086-8982-7e44ca6bdc4d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_other_tenants_public_share(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_shares_v2_client.delete_share,
                          self.share['id'])

    @decorators.idempotent_id('1f9e5d84-0885-4a4b-9196-9031a1c01508')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_of_other_tenants_public_share(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_shares_v2_client.set_metadata,
                          self.share['id'],
                          {'key': 'value'})

    @decorators.idempotent_id('fed7a935-9699-43a1-854e-67b61ba6233e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_metadata_of_other_tenants_public_share(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_shares_v2_client.update_all_metadata,
                          self.share['id'],
                          {'key': 'value'})

    @decorators.idempotent_id('bd62adeb-73c2-4b04-8812-80b479cd5c3b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_metadata_of_other_tenants_public_share(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.alt_shares_v2_client.delete_metadata,
                          self.share['id'],
                          'key')



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.api import test_security_services


class SecurityServiceAdminTest(
        base.BaseSharesAdminTest,
        test_security_services.SecurityServiceListMixin):

    def setUp(self):
        super(SecurityServiceAdminTest, self).setUp()
        ss_ldap_data = {
            'name': 'ss_ldap',
            'dns_ip': '1.1.1.1',
            'server': 'fake_server_1',
            'domain': 'fake_domain_1',
            'user': 'fake_user',
            'password': 'pass',
        }
        ss_kerberos_data = {
            'name': 'ss_kerberos',
            'dns_ip': '2.2.2.2',
            'server': 'fake_server_2',
            'domain': 'fake_domain_2',
            'user': 'test_user',
            'password': 'word',
        }
        self.ss_ldap = self.create_security_service('ldap', **ss_ldap_data)
        self.ss_kerberos = self.create_security_service(
            'kerberos',
            **ss_kerberos_data)

    @decorators.idempotent_id('6f843a0a-21da-4a83-ae3b-3831f99b3678')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_security_services_all_tenants(self):
        listed = self.shares_client.list_security_services(
            params={'all_tenants': 1})['security_services']
        self.assertTrue(any(self.ss_ldap['id'] == ss['id'] for ss in listed))
        self.assertTrue(any(self.ss_kerberos['id'] == ss['id']
                            for ss in listed))

        keys = ["name", "id", "status", "type", ]
        [self.assertIn(key, s_s.keys()) for s_s in listed for key in keys]

    @decorators.idempotent_id('fe515fa7-0275-4190-adc7-5321e1047dad')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_security_services_invalid_filters(self):
        listed = self.shares_client.list_security_services(
            params={'fake_opt': 'some_value'})['security_services']
        self.assertEqual(0, len(listed))



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacExtraSpecsTests(rbac_base.ShareRbacBaseTests,
                               metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacExtraSpecsTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.admin_shares_v2_client = (
            cls.os_project_admin.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        super(ShareRbacExtraSpecsTests, cls).resource_setup()
        cls.extra_specs = {u'key': u'value'}
        cls.share_type = cls.create_share_type()

    @abc.abstractmethod
    def test_create_share_type_extra_specs(self):
        pass

    @abc.abstractmethod
    def test_get_share_type_extra_specs(self):
        pass

    @abc.abstractmethod
    def test_update_share_type_extra_spec(self):
        pass

    @abc.abstractmethod
    def test_delete_share_type_extra_spec(self):
        pass


class ProjectAdminTests(ShareRbacExtraSpecsTests, base.BaseSharesTest):

    credentials = ['project_admin']

    @classmethod
    def setup_clients(cls):
        super(ProjectAdminTests, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('d51817f4-b186-4eca-8779-f3c36dcd98e7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_type_extra_specs(self):
        self.do_request(
            'create_share_type_extra_specs', expected_status=200,
            share_type_id=self.share_type['id'], extra_specs=self.extra_specs)
        self.addCleanup(
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            self.share_type['id'], extra_spec_name='key')

    @decorators.idempotent_id('d4deeb0b-8765-4487-9f1f-9f10088934dd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_share_type_extra_specs(self):
        self.do_request(
            'get_share_type_extra_specs', expected_status=200,
            share_type_id=self.share_type['id'])

    @decorators.idempotent_id('bb27641b-5249-4343-bb08-5a123f31b9f1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_share_type_extra_spec(self):
        self.do_request(
            'update_share_type_extra_spec', expected_status=200,
            share_type_id=self.share_type['id'], spec_name='key',
            spec_value='value_updated')

    @decorators.idempotent_id('81d59322-8ec1-4f32-a50d-2fedd1cca655')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_delete_share_type_extra_spec(self):
        self.admin_shares_v2_client.create_share_type_extra_specs(
            self.share_type['id'], self.extra_specs)
        self.do_request(
            'delete_share_type_extra_spec', expected_status=202,
            share_type_id=self.share_type['id'], extra_spec_name='key')


class ProjectMemberTests(ShareRbacExtraSpecsTests, base.BaseSharesTest):

    credentials = ['project_member', 'project_admin']

    @decorators.idempotent_id('446c1e7c-5ca2-46f5-b2f3-6417152e5bf8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_type_extra_specs(self):
        self.do_request(
            'create_share_type_extra_specs', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], extra_specs=self.extra_specs)

    @decorators.idempotent_id('aabdf0c1-8b68-4c40-a542-cc1dbd039279')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_share_type_extra_specs(self):
        self.do_request(
            'get_share_type_extra_specs', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'])

    @decorators.idempotent_id('3629f91c-ad21-4321-acd9-7fca92a4721a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_type_extra_spec(self):
        self.do_request(
            'update_share_type_extra_spec', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], spec_name='key',
            spec_value='value_updated')

    @decorators.idempotent_id('0bee97ab-b406-481c-9b8b-4813fb9f49dc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_type_extra_spec(self):
        self.admin_shares_v2_client.create_share_type_extra_specs(
            self.share_type['id'], self.extra_specs)
        self.do_request(
            'delete_share_type_extra_spec', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], extra_spec_name='key')
        self.addCleanup(
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            self.share_type['id'], extra_spec_name='key')


class ProjectReaderTests(ProjectMemberTests):

    credentials = ['project_reader', 'project_admin']

    @decorators.idempotent_id('da80b823-9a96-45c3-8f86-e9f7fc5ad2c6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_type_extra_specs(self):
        super(ProjectReaderTests, self).test_create_share_type_extra_specs()

    @decorators.idempotent_id('78989220-22dc-46d2-b83b-63f070988eed')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_share_type_extra_specs(self):
        super(ProjectReaderTests, self).test_get_share_type_extra_specs()

    @decorators.idempotent_id('c77173d4-b90c-4edf-a9b6-a26c81aaec42')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_share_type_extra_spec(self):
        super(ProjectReaderTests, self).test_update_share_type_extra_spec()

    @decorators.idempotent_id('46fc8e62-b987-444f-ab9f-cd86a8960156')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_share_type_extra_spec(self):
        super(ProjectReaderTests, self).test_delete_share_type_extra_spec()



# Copyright 2015 Goutham Pacha Ravi
# Copyright 2015 Clinton Knight
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base

CONF = config.CONF

API_MICROVERSIONS_HEADER_LOWER = 'x-openstack-manila-api-version'
API_MICROVERSIONS_HEADER = 'X-OpenStack-Manila-API-Version'
_MIN_API_VERSION = CONF.share.min_api_microversion
_MAX_API_VERSION = CONF.share.max_api_microversion


class MicroversionsTest(base.BaseSharesTest):
    """Request and validate REST API Microversions.

    Sends HTTP GET requests to the version API to validate microversions.
    """

    @decorators.idempotent_id('1b20943e-a398-49dd-a69d-513ce6ea01b4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_root_version(self):

        resp, resp_body = self.shares_v2_client.send_microversion_request()

        self.assertEqual(300, resp.status)

        version_list = resp_body['versions']
        ids = [v['id'] for v in version_list]
        self.assertEqual({'v1.0', 'v2.0'}, set(ids))

        self.assertNotIn(API_MICROVERSIONS_HEADER_LOWER, resp)
        self.assertNotIn('vary', resp)

        v1 = [v for v in version_list if v['id'] == 'v1.0'][0]
        self.assertEqual('', v1.get('min_version'))
        self.assertEqual('', v1.get('version'))

        v2 = [v for v in version_list if v['id'] == 'v2.0'][0]
        self.assertEqual(_MIN_API_VERSION, v2.get('min_version'))
        self.assertNotIn(v2.get('version'), [None, ''])

    @decorators.idempotent_id('6ccaae5f-2382-45a5-a844-6d31837beba8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v1_no_version(self):

        resp, resp_body = self.shares_v2_client.send_microversion_request(
            script_name='v1')

        self.assertEqual(200, resp.status)

        version_list = resp_body['versions']
        ids = [v['id'] for v in version_list]
        self.assertEqual({'v1.0'}, set(ids))

        self.assertEqual('1.0', resp.get(API_MICROVERSIONS_HEADER_LOWER))
        self.assertIn(API_MICROVERSIONS_HEADER,
                      resp.get('vary', '').split(','))
        self.assertEqual('', version_list[0].get('min_version'))
        self.assertEqual('', version_list[0].get('version'))

    @decorators.idempotent_id('7b7e4ddf-08bb-4764-a94f-5f377da3b2cb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v1_with_version(self):

        resp, resp_body = self.shares_v2_client.send_microversion_request(
            script_name='v1', version='5.0')

        self.assertEqual(200, resp.status)

        version_list = resp_body['versions']
        ids = [v['id'] for v in version_list]
        self.assertEqual({'v1.0'}, set(ids))

        self.assertEqual('1.0', resp.get(API_MICROVERSIONS_HEADER_LOWER))
        self.assertIn(API_MICROVERSIONS_HEADER,
                      resp.get('vary', '').split(','))
        self.assertEqual('', version_list[0].get('min_version'))
        self.assertEqual('', version_list[0].get('version'))

    @decorators.idempotent_id('9a8bd7fd-f9d0-4fc6-8e1c-0178d87ec7c1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v2_no_version(self):

        resp, resp_body = self.shares_v2_client.send_microversion_request(
            script_name='v2')

        self.assertEqual(200, resp.status)

        version_list = resp_body['versions']
        ids = [v['id'] for v in version_list]
        self.assertEqual({'v2.0'}, set(ids))

        self.assertEqual(_MIN_API_VERSION,
                         resp.get(API_MICROVERSIONS_HEADER_LOWER))
        self.assertIn(API_MICROVERSIONS_HEADER,
                      resp.get('vary', '').split(','))
        self.assertEqual(_MIN_API_VERSION, version_list[0].get('min_version'))
        self.assertNotIn(version_list[0].get('version'), [None, ''])

    @decorators.idempotent_id('101574fd-9d47-4bd6-97f6-cde219f089ca')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v2_min_version(self):

        resp, resp_body = self.shares_v2_client.send_microversion_request(
            script_name='v2', version=_MIN_API_VERSION)

        self.assertEqual(200, resp.status)

        version_list = resp_body['versions']
        ids = [v['id'] for v in version_list]
        self.assertEqual({'v2.0'}, set(ids))

        self.assertEqual(_MIN_API_VERSION,
                         resp.get(API_MICROVERSIONS_HEADER_LOWER))
        self.assertIn(API_MICROVERSIONS_HEADER,
                      resp.get('vary', '').split(','))
        self.assertEqual(_MIN_API_VERSION, version_list[0].get('min_version'))
        self.assertNotIn(version_list[0].get('version'), [None, ''])

    @decorators.idempotent_id('e0766a36-b57c-46f8-b474-a38b74f3b9ed')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v2_max_version(self):

        resp, resp_body = self.shares_v2_client.send_microversion_request(
            script_name='v2', version=_MAX_API_VERSION)

        self.assertEqual(200, resp.status)

        version_list = resp_body['versions']
        ids = [v['id'] for v in version_list]
        self.assertEqual({'v2.0'}, set(ids))

        self.assertEqual(_MAX_API_VERSION,
                         resp.get(API_MICROVERSIONS_HEADER_LOWER))
        self.assertIn(API_MICROVERSIONS_HEADER,
                      resp.get('vary', '').split(','))
        self.assertEqual(_MIN_API_VERSION, version_list[0].get('min_version'))
        self.assertNotIn(version_list[0].get('version'), [None, ''])

    @decorators.idempotent_id('038ff428-edde-43ff-ae29-1b18676b61d5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v2_invalid_version(self):

        resp, _ = self.shares_v2_client.send_microversion_request(
            script_name='v2', version='1.2.1')

        self.assertEqual(400, resp.status)

    @decorators.idempotent_id('0d82fc0b-d24d-4d07-ab43-bd0895ec0c4c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_microversions_v2_unacceptable_version(self):

        # First get max version from the server
        resp, resp_body = self.shares_v2_client.send_microversion_request(
            script_name='v2')

        self.assertEqual(200, resp.status)

        version_list = resp_body['versions']
        latest_version = version_list[0].get('version')
        major, minor = [int(ver) for ver in latest_version.split(".")]
        next_version = ('%s.%s' % (major + 1, minor + 1))

        # Request a version that is too high
        resp, _ = self.shares_v2_client.send_microversion_request(
            script_name='v2', version=next_version)

        self.assertEqual(406, resp.status)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.api import test_replication_negative as \
    rep_neg_test
from manila_tempest_tests import utils

CONF = config.CONF
PRE_SHARE_GROUPS_MICROVERSION = "2.39"
SHARE_GROUPS_MICROVERSION = "2.40"
PRE_SHARE_REPLICA_QUOTAS_MICROVERSION = "2.52"
SHARE_REPLICA_QUOTAS_MICROVERSION = "2.53"


@ddt.ddt
class SharesAdminQuotasNegativeTest(base.BaseSharesAdminTest):

    # We want to force some fresh projects for this test class, since we'll be
    # manipulating project quotas - and any pre-existing projects may have
    # resources, quotas and the like that might interfere with our test cases.
    force_tenant_isolation = True

    @classmethod
    def skip_checks(cls):
        super(SharesAdminQuotasNegativeTest, cls).skip_checks()
        if not CONF.auth.use_dynamic_credentials:
            raise cls.skipException('Dynamic credentials are required')
        if not CONF.share.run_quota_tests:
            msg = "Quota tests are disabled."
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(SharesAdminQuotasNegativeTest, cls).resource_setup()
        cls.client = cls.shares_v2_client
        cls.user_id = cls.client.user_id
        cls.tenant_id = cls.client.tenant_id
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share group type
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']

    @decorators.idempotent_id('c7174059-7172-4cc8-9121-aefe509ef14c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_quotas_with_empty_tenant_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_quotas, "")

    @decorators.idempotent_id('0fbcbfad-fdb5-42d6-b005-041dc4ddea64')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_reset_quotas_with_empty_tenant_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.client.reset_quotas, "")

    @ddt.data(
        {"shares": -2},
        {"snapshots": -2},
        {"gigabytes": -2},
        {"snapshot_gigabytes": -2},
        {"share_networks": -2},
    )
    @decorators.idempotent_id('07d3e69a-7cda-4ca7-9fea-c32f6830fdd3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_update_quota_with_wrong_data(self, kwargs):
        # -1 is acceptable value as unlimited
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          **kwargs)

    @ddt.data(
        {"share_groups": -2},
        {"share_group_snapshots": -2},
    )
    @decorators.idempotent_id('b11c5d5f-2b58-4a5f-8469-b22ea51709c0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_update_sg_quota_with_wrong_data(self, kwargs):
        # -1 is acceptable value as unlimited
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          **kwargs)

    @ddt.data(
        {"share_replicas": -2},
        {"replica_gigabytes": -2},
    )
    @decorators.idempotent_id('d070ccc6-6685-4f49-a8e5-9b891790881e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_update_replica_quotas_wrong_data(self, kwargs):
        # -1 is acceptable value as unlimited
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          **kwargs)

    @decorators.idempotent_id('75d39eda-a2b5-4271-a61d-9e2c86370b3e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_create_share_with_size_bigger_than_quota(self):
        quotas = self.client.show_quotas(self.tenant_id)['quota_set']
        overquota = int(quotas['gigabytes']) + 2

        # try schedule share with size, bigger than gigabytes quota
        self.assertRaises(lib_exc.OverLimit,
                          self.create_share,
                          size=overquota)

    @decorators.idempotent_id('37dd40a8-375e-454b-8b80-229cb0eecb01')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @testtools.skipUnless(
        CONF.share.run_share_group_tests, 'Share Group tests disabled.')
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_create_share_group_with_exceeding_quota_limit(self):
        self.update_quotas(self.tenant_id, share_groups=0)

        # Try schedule share group creation
        self.assertRaises(lib_exc.OverLimit,
                          self.create_share_group,
                          share_group_type_id=self.share_group_type_id,
                          share_type_ids=[self.share_type_id],
                          cleanup_in_class=False)

    @decorators.idempotent_id('e039535c-dc4e-497a-ac09-30b3395ba95b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_user_quota_shares_bigger_than_tenant_quota(self):

        # get current quotas for tenant
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # try set user quota for shares bigger than tenant quota
        bigger_value = int(tenant_quotas["shares"]) + 2
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          user_id=self.user_id,
                          force=False,
                          shares=bigger_value)

    @decorators.idempotent_id('57495588-d645-402e-9b89-0ab63a58ce3e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_user_quota_snaps_bigger_than_tenant_quota(self):

        # get current quotas for tenant
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # try set user quota for snapshots bigger than tenant quota
        bigger_value = int(tenant_quotas["snapshots"]) + 2
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          user_id=self.user_id,
                          force=False,
                          snapshots=bigger_value)

    @decorators.idempotent_id('6187050f-f262-48cf-bdd6-32982f860fba')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_user_quota_gigabytes_bigger_than_tenant_quota(self):

        # get current quotas for tenant
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # try set user quota for gigabytes bigger than tenant quota
        bigger_value = int(tenant_quotas["gigabytes"]) + 2
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          user_id=self.user_id,
                          force=False,
                          gigabytes=bigger_value)

    @decorators.idempotent_id('8eb4aed8-c239-49d3-aee1-d63d043aa3d1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_user_quota_snap_gigabytes_bigger_than_tenant_quota(self):
        # get current quotas for tenant
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # try set user quota for snapshot gigabytes bigger than tenant quota
        bigger_value = int(tenant_quotas["snapshot_gigabytes"]) + 2
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          user_id=self.user_id,
                          force=False,
                          snapshot_gigabytes=bigger_value)

    @decorators.idempotent_id('a272a24b-21a6-4f6c-917c-cbe103203a11')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_user_quota_share_networks_bigger_than_tenant_quota(self):

        # get current quotas for tenant
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # try set user quota for share_networks bigger than tenant quota
        bigger_value = int(tenant_quotas["share_networks"]) + 2
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          user_id=self.user_id,
                          force=False,
                          share_networks=bigger_value)

    @ddt.data("share_replicas", "replica_gigabytes")
    @decorators.idempotent_id('40dabf7d-98da-48a9-bb62-285098d5acb4')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_try_set_user_quota_replicas_bigger_than_tenant_quota(self, key):
        # get current quotas for tenant
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # try set user quota for snapshots bigger than tenant quota
        bigger_value = int(tenant_quotas[key]) + 2
        kwargs = {key: bigger_value}
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          user_id=self.user_id,
                          force=False,
                          **kwargs)

    @ddt.data(
        ('quota-sets', '2.0', 'show_quotas'),
        ('quota-sets', '2.0', 'default_quotas'),
        ('quota-sets', '2.0', 'reset_quotas'),
        ('quota-sets', '2.0', 'update_quotas'),
        ('quota-sets', '2.6', 'show_quotas'),
        ('quota-sets', '2.6', 'default_quotas'),
        ('quota-sets', '2.6', 'reset_quotas'),
        ('quota-sets', '2.6', 'update_quotas'),
        ('os-quota-sets', '2.7', 'show_quotas'),
        ('os-quota-sets', '2.7', 'default_quotas'),
        ('os-quota-sets', '2.7', 'reset_quotas'),
        ('os-quota-sets', '2.7', 'update_quotas'),
    )
    @ddt.unpack
    @decorators.idempotent_id('ed38ab0a-694c-48ea-bce5-5c264f485d5b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.7")
    def test_show_quotas_with_wrong_versions(self, url, version, method_name):
        self.assertRaises(lib_exc.NotFound,
                          getattr(self.client, method_name),
                          self.tenant_id,
                          version=version,
                          url=url)

    @decorators.idempotent_id('50af5d8c-0b30-4e8d-93bf-bb975db58516')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_quota_detail_with_wrong_versions(self):
        version = '2.24'
        url = 'quota-sets'

        self.assertRaises(lib_exc.NotFound,
                          self.client.detail_quotas,
                          self.tenant_id,
                          version=version,
                          url=url)

    @ddt.data('show', 'reset', 'update')
    @decorators.idempotent_id('cf45eb7d-7330-4b2d-8214-e4149eb4a398')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_share_type_quotas_using_nonexistent_share_type(self, op):

        kwargs = {"share_type": "fake_nonexistent_share_type"}
        if op == 'update':
            tenant_quotas = self.client.show_quotas(
                self.tenant_id)['quota_set']
            kwargs['shares'] = tenant_quotas['shares']

        self.assertRaises(lib_exc.NotFound,
                          getattr(self.client, op + '_quotas'),
                          self.tenant_id,
                          **kwargs)

    @ddt.data('id', 'name')
    @decorators.idempotent_id('2ba641a1-100b-417e-80e2-d3f717fd3c7c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_try_update_share_type_quota_for_share_networks(self, key):
        share_type = self.create_share_type()
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        # Try to set 'share_networks' quota for share type
        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          share_type=share_type[key],
                          share_networks=int(tenant_quotas["share_networks"]))

    @ddt.data('share_groups', 'share_group_snapshots')
    @decorators.idempotent_id('5eb6ce15-1172-4bcb-9c7b-91543bf714e8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_try_update_share_type_quota_for_share_groups(self, quota_name):
        share_type = self.create_share_type()
        tenant_quotas = self.client.show_quotas(self.tenant_id)['quota_set']

        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          share_type=share_type["name"],
                          **{quota_name: int(tenant_quotas[quota_name])})

    @ddt.data('share_groups', 'share_group_snapshots')
    @decorators.idempotent_id('1b504c74-2ce9-40f6-87fb-9e643b1b5906')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(PRE_SHARE_GROUPS_MICROVERSION)
    @utils.skip_if_microversion_not_supported(SHARE_GROUPS_MICROVERSION)
    def test_share_group_quotas_using_too_old_microversion(self, quota_key):
        tenant_quotas = self.client.show_quotas(
            self.tenant_id, version=SHARE_GROUPS_MICROVERSION)['quota_set']
        kwargs = {
            "version": PRE_SHARE_GROUPS_MICROVERSION,
            quota_key: tenant_quotas[quota_key],
        }

        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_quotas,
                          self.tenant_id,
                          **kwargs)

    @ddt.data("share_replicas", "replica_gigabytes")
    @decorators.idempotent_id('66f22d42-37bc-4f9b-8e0b-a679341e1e88')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported(
        SHARE_REPLICA_QUOTAS_MICROVERSION)
    def test_share_replica_quotas_using_too_old_microversion(self, quota_key):
        tenant_quotas = self.client.show_quotas(
            self.tenant_id,
            version=SHARE_REPLICA_QUOTAS_MICROVERSION)['quota_set']
        kwargs = {
            "version": PRE_SHARE_REPLICA_QUOTAS_MICROVERSION,
            quota_key: tenant_quotas[quota_key],
        }

        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          **kwargs)

    @ddt.data('show', 'reset', 'update')
    @decorators.idempotent_id('acc609c2-f314-4540-984c-33e93d048f6c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.38")
    def test_share_type_quotas_using_too_old_microversion(self, op):
        share_type = self.create_share_type()
        kwargs = {"version": "2.38", "share_type": share_type["name"]}
        if op == 'update':
            tenant_quotas = self.client.show_quotas(
                self.tenant_id)['quota_set']
            kwargs['shares'] = tenant_quotas['shares']

        self.assertRaises(lib_exc.BadRequest,
                          getattr(self.client, op + '_quotas'),
                          self.tenant_id,
                          **kwargs)

    @ddt.data('show', 'reset', 'update')
    @decorators.idempotent_id('719768d1-d313-40e9-9127-c5777840ecbd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_quotas_providing_share_type_and_user_id(self, op):
        share_type = self.create_share_type()
        kwargs = {"share_type": share_type["name"], "user_id": self.user_id}
        if op == 'update':
            tenant_quotas = self.client.show_quotas(
                self.tenant_id)['quota_set']
            kwargs['shares'] = tenant_quotas['shares']

        self.assertRaises(lib_exc.BadRequest,
                          getattr(self.client, op + '_quotas'),
                          self.tenant_id,
                          **kwargs)

    @ddt.data(11, -1)
    @decorators.idempotent_id('82256511-aa46-4b99-a6e5-8b400534e96d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @utils.skip_if_microversion_not_supported("2.39")
    def test_update_share_type_quotas_bigger_than_project_quota(self, st_q):
        share_type = self.create_share_type()
        self.update_quotas(self.tenant_id, shares=10)

        self.assertRaises(lib_exc.BadRequest,
                          self.update_quotas,
                          self.tenant_id,
                          share_type=share_type['name'],
                          force=False,
                          shares=st_q)

    @decorators.idempotent_id('6396daab-ba73-4140-8a0a-8eef0a01804d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share_over_quota_limit(self):
        self.create_share(share_type_id=self.share_type_id)

        updated_quota = self.update_quotas(self.tenant_id, shares=1)
        self.assertEqual(1, updated_quota['shares'])

        self.assertRaises(lib_exc.OverLimit,
                          self.create_share,
                          share_type_id=self.share_type_id)

    @decorators.idempotent_id('a2267f4d-63ef-4631-a01d-3723707e5516')
    @testtools.skipUnless(
        CONF.share.run_snapshot_tests, 'Snapshot tests are disabled.')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_snapshot_over_quota_limit(self):
        extra_specs = {'snapshot_support': True}
        share_type = self.create_share_type(extra_specs=extra_specs)
        share = self.create_share(share_type_id=share_type['id'])

        # Update snapshot quota
        self.update_quotas(self.tenant_id, snapshots=1)

        # Create share from updated snapshot, wait for status 'available'
        self.create_snapshot_wait_for_active(share['id'])

        self.assertRaises(lib_exc.OverLimit,
                          self.create_snapshot_wait_for_active,
                          share['id'])


@ddt.ddt
class ReplicaQuotasNegativeTest(rep_neg_test.ReplicationNegativeBase):

    # We want to force some fresh projects for this test class, since we'll be
    # manipulating project quotas - and any pre-existing projects may have
    # resources, quotas and the like that might interfere with our test cases.
    force_tenant_isolation = True

    @classmethod
    def resource_setup(cls):
        super(ReplicaQuotasNegativeTest, cls).resource_setup()
        cls.client = cls.shares_v2_client
        cls.user_id = cls.client.user_id
        cls.tenant_id = cls.client.tenant_id

    @classmethod
    def skip_checks(cls):
        super(ReplicaQuotasNegativeTest, cls).skip_checks()
        if not CONF.auth.use_dynamic_credentials:
            raise cls.skipException('Dynamic credentials are required')
        if not CONF.share.run_quota_tests:
            msg = "Quota tests are disabled."
            raise cls.skipException(msg)
        utils.check_skip_if_microversion_not_supported(
            SHARE_REPLICA_QUOTAS_MICROVERSION)

    def _modify_quotas_for_test(self, quota_key, new_limit):
        kwargs = {quota_key: new_limit}

        # Update the current quotas
        self.update_quotas(self.tenant_id, client=self.admin_client, **kwargs)

        # Get the updated quotas and add a cleanup
        updated_quota = self.admin_client.show_quotas(
            self.tenant_id)['quota_set']

        # Make sure that the new value was properly set
        self.assertEqual(new_limit, updated_quota[quota_key])

    @decorators.idempotent_id('ad19eaa5-bacd-4bc5-9592-622b0c856cdd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(('share_replicas', 2), ('replica_gigabytes', None))
    @ddt.unpack
    def test_create_replica_over_replica_limit(self, quota_key, new_limit):
        # Define the quota values to be updated
        new_limit = (int(self.share1['size'] * 2)
                     if quota_key == 'replica_gigabytes' else new_limit)

        # Create an inactive share replica
        self.create_share_replica(
            self.share1["id"], self.replica_zone, cleanup_in_class=False)

        # Modify the quota limit for this test
        self._modify_quotas_for_test(quota_key, new_limit)

        # Make sure that the request to create a third one will fail
        self.assertRaises(lib_exc.OverLimit,
                          self.create_share_replica,
                          self.share1['id'],
                          availability_zone=self.replica_zone)

    @decorators.idempotent_id('0006f1ff-69c0-40b7-8437-55cc0a08a195')
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_extend_replica_over_limit(self):
        # Define the quota values to be updated
        quota_key = 'replica_gigabytes'

        # Modify the quota limit for this test
        self._modify_quotas_for_test(quota_key, new_limit=self.share1['size'])

        new_size = self.share1['size'] + 1

        # Make sure that the request to create a third one will fail
        self.assertRaises(lib_exc.OverLimit,
                          self.client.extend_share,
                          self.share1['id'],
                          new_size)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base
from manila_tempest_tests import utils

CONF = config.CONF


class ShareRbacRulesTests(rbac_base.ShareRbacBaseTests, metaclass=abc.ABCMeta):

    @classmethod
    def skip_checks(cls):
        super(ShareRbacRulesTests, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    @classmethod
    def setup_clients(cls):
        super(ShareRbacRulesTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        super(ShareRbacRulesTests, cls).resource_setup()
        cls.metadata = {u'key': u'value'}

    def access(self, share_id, access_type, access_to, access_level='rw'):
        access = {}
        access['share_id'] = share_id
        access['access_type'] = access_type
        access['access_to'] = access_to
        access['access_level'] = access_level
        return access

    def allow_access(self, client, share_id, access_type, access_to,
                     access_level='rw', metadata=None, status='active',
                     cleanup=True):

        kwargs = {
            'access_type': access_type,
            'access_to': access_to,
            'access_level': access_level,
            'metadata': metadata
        }

        rule = client.create_access_rule(share_id, **kwargs)['access']
        waiters.wait_for_resource_status(
            client, share_id, status, resource_name='access_rule',
            rule_id=rule['id'])
        if cleanup:
            self.addCleanup(
                client.wait_for_resource_deletion, rule_id=rule['id'],
                share_id=share_id)
            self.addCleanup(client.delete_access_rule, share_id, rule['id'])
        return rule

    @abc.abstractmethod
    def test_grant_access_rule(self):
        pass

    @abc.abstractmethod
    def test_get_access(self):
        pass

    @abc.abstractmethod
    def test_list_access(self):
        pass

    @abc.abstractmethod
    def test_delete_access(self):
        pass

    @abc.abstractmethod
    def test_update_access_rule_metadata(self):
        pass

    @abc.abstractmethod
    def test_delete_access_rule_metadata(self):
        pass


class TestProjectAdminTestsNFS(ShareRbacRulesTests, base.BaseSharesTest):
    credentials = ['project_admin', 'project_alt_member']
    protocol = 'nfs'

    @classmethod
    def setup_clients(cls):
        super(TestProjectAdminTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @classmethod
    def resource_setup(cls):
        super(TestProjectAdminTestsNFS, cls).resource_setup()
        share_type = cls.get_share_type()
        cls.share = cls.create_share(cls.client, share_type['id'])
        cls.alt_share = cls.create_share(
            cls.alt_project_share_v2_client, share_type['id'])

    @decorators.idempotent_id('5b6897d1-4b2a-490c-990e-941ea4893f47')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'get_access_rule', expected_status=200, access_id=access['id'])

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'get_access_rule', expected_status=200, access_id=alt_access['id'])

    @decorators.idempotent_id('f8e9a2bb-ccff-4fc5-8d61-2930f87406cd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to)
        access_list = self.do_request(
            'list_access_rules', expected_status=200,
            share_id=self.share['id'])['access_list'][0]['id']

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        alt_access_list = self.do_request(
            'list_access_rules', expected_status=200,
            share_id=self.share['id'])['access_list'][0]['id']

        self.assertIn(access['id'], access_list)
        self.assertNotIn(alt_access['id'], alt_access_list)

    @decorators.idempotent_id('b4d7a91c-a75e-4ad9-93cb-8e5234fea97a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_grant_access_rule(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.do_request(
            'create_access_rule', expected_status=200,
            **self.access(self.share['id'], access_type, access_to))['access']
        waiters.wait_for_resource_status(
            self.client, self.share["id"], "active",
            resource_name='access_rule', rule_id=access["id"])
        self.addCleanup(
            self.client.wait_for_resource_deletion, rule_id=access['id'],
            share_id=self.share['id'])
        self.addCleanup(
            self.client.delete_access_rule, self.share['id'], access['id'])

        alt_access = self.do_request(
            'create_access_rule', expected_status=200,
            **self.access(
                self.alt_share['id'], access_type, access_to))['access']
        waiters.wait_for_resource_status(
            self.client, self.alt_share["id"], "active",
            resource_name='access_rule', rule_id=alt_access["id"])
        self.addCleanup(
            self.client.wait_for_resource_deletion, rule_id=alt_access['id'],
            share_id=self.alt_share['id'])
        self.addCleanup(
            self.client.delete_access_rule, self.alt_share['id'],
            alt_access['id'])

    @decorators.idempotent_id('e24d7018-cb49-4306-9947-716b4e4250c5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type,
            access_to=access_to, cleanup=False)
        self.do_request(
            'delete_access_rule', expected_status=202,
            share_id=self.share['id'], rule_id=access['id'])
        self.client.wait_for_resource_deletion(
            rule_id=access['id'], share_id=self.share['id'])

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to,
            cleanup=False)
        self.do_request(
            'delete_access_rule', expected_status=202,
            share_id=self.alt_share['id'], rule_id=alt_access['id'])
        self.client.wait_for_resource_deletion(
            rule_id=alt_access['id'], share_id=self.alt_share['id'])

    @decorators.idempotent_id('ffc07445-d0d1-4bf9-9fbc-4f409d48bccd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_update_access_rule_metadata(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'update_access_metadata', expected_status=200,
            access_id=access['id'], metadata=self.metadata)

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'update_access_metadata', expected_status=200,
            access_id=alt_access['id'], metadata=self.metadata)

    @decorators.idempotent_id('fd580d91-1d8d-4dd0-8484-01c412ddb768')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_access_rule_metadata(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to,
            metadata=self.metadata)
        self.do_request(
            'delete_access_metadata', expected_status=200,
            access_id=access['id'], key='key')

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to,
            metadata=self.metadata)
        self.do_request(
            'delete_access_metadata', expected_status=200,
            access_id=alt_access['id'], key='key')


class TestProjectMemberTestsNFS(ShareRbacRulesTests, base.BaseSharesTest):
    credentials = ['project_member', 'project_alt_member']
    protocol = 'nfs'

    @classmethod
    def resource_setup(cls):
        super(TestProjectMemberTestsNFS, cls).resource_setup()
        share_type = cls.get_share_type()
        share_client = getattr(cls, 'share_member_client', cls.client)
        cls.share = cls.create_share(share_client, share_type['id'])
        cls.alt_share = cls.create_share(
            cls.alt_project_share_v2_client, share_type['id'])

    @decorators.idempotent_id('de643909-88a2-470b-8a14-0417696ec451')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        share_client = getattr(self, 'share_member_client', self.client)
        access = self.allow_access(
            share_client, self.share['id'], access_type=access_type,
            access_to=access_to)
        self.do_request(
            'get_access_rule', expected_status=200, access_id=access['id'])

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'get_access_rule', expected_status=lib_exc.NotFound,
            access_id=alt_access['id'])

    @decorators.idempotent_id('7c6c4262-5095-4cd7-9d9c-8064009a9055')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        share_client = getattr(self, 'share_member_client', self.client)
        access = self.allow_access(
            share_client, self.share['id'], access_type=access_type,
            access_to=access_to)
        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)

        access_list = self.do_request(
            'list_access_rules', expected_status=200,
            share_id=self.share['id'])['access_list']
        access_id_list = [
            s['id'] for s in access_list
        ]

        self.assertIn(access['id'], access_id_list)
        self.assertNotIn(alt_access['id'], access_id_list)

    @decorators.idempotent_id('61cf6f6c-5d7c-48d7-9d5a-e6ea288afdbc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_grant_access_rule(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        share_client = getattr(self, 'share_member_client', self.client)
        access = self.do_request(
            'create_access_rule', client=share_client, expected_status=200,
            **self.access(self.share['id'], access_type, access_to))['access']
        waiters.wait_for_resource_status(
            share_client, self.share["id"], "active",
            resource_name='access_rule', rule_id=access["id"])
        self.addCleanup(
            self.client.wait_for_resource_deletion, rule_id=access['id'],
            share_id=self.share['id'])
        self.addCleanup(
            self.client.delete_access_rule, self.share['id'], access['id'])

        self.do_request(
            'create_access_rule', client=share_client,
            expected_status=lib_exc.NotFound,
            **self.access(self.alt_share['id'], access_type, access_to))

    @decorators.idempotent_id('8665d1b1-de4c-42d4-93ff-8dc6d2b73a2d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        share_client = getattr(self, 'share_member_client', self.client)
        access = self.allow_access(
            share_client, self.share['id'], access_type=access_type,
            access_to=access_to, cleanup=False)
        self.do_request(
            'delete_access_rule', expected_status=202,
            share_id=self.share['id'], rule_id=access['id'])
        self.client.wait_for_resource_deletion(
            rule_id=access['id'], share_id=self.share['id'])

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'delete_access_rule', expected_status=lib_exc.NotFound,
            share_id=self.alt_share['id'], rule_id=alt_access['id'])

    @decorators.idempotent_id('c5e84362-6075-425b-bfa3-898abfd9d5a0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_access_rule_metadata(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        share_client = getattr(self, 'share_member_client', self.client)
        access = self.allow_access(
            share_client, self.share['id'], access_type=access_type,
            access_to=access_to)
        self.do_request(
            'update_access_metadata', expected_status=200,
            access_id=access['id'], metadata=self.metadata)

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'update_access_metadata', expected_status=lib_exc.NotFound,
            access_id=alt_access['id'], metadata=self.metadata)

    @decorators.idempotent_id('abb17315-6510-4b6e-ae6c-dd99a6088954')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_access_rule_metadata(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        share_client = getattr(self, 'share_member_client', self.client)
        access = self.allow_access(
            share_client, self.share['id'], access_type=access_type,
            access_to=access_to, metadata=self.metadata)
        self.do_request(
            'delete_access_metadata', expected_status=200,
            access_id=access['id'], key='key')

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to,
            metadata=self.metadata)
        self.do_request(
            'delete_access_metadata', expected_status=lib_exc.NotFound,
            access_id=alt_access['id'], key='key')


class TestProjectReaderTestsNFS(TestProjectMemberTestsNFS):
    """Test suite for basic share access rule operations by reader user

    In order to test certain share operations we must create a share
    resource for this. Since reader user is limited in resources creation, we
    are forced to use admin credentials, so we can test other share
    operations. In this class we use admin user to create a member user within
    reader project. That way we can perform a reader actions on this resource.
    """

    credentials = ['project_reader', 'project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(TestProjectReaderTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('0eec0f05-f2f3-4500-9d9e-1b77ebc476c2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_access(self):
        super(TestProjectReaderTestsNFS, self).test_get_access()

    @decorators.idempotent_id('9ddc26b6-f8bf-45d9-a2c6-a9eec9bfb8d2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_access(self):
        super(TestProjectReaderTestsNFS, self).test_list_access()

    @decorators.idempotent_id('ace870f9-af91-4259-8760-dc7d7107b7ff')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_grant_access_rule(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        self.do_request(
            'create_access_rule', expected_status=lib_exc.Forbidden,
            **self.access(self.share['id'], access_type, access_to))

        self.do_request(
            'create_access_rule', expected_status=lib_exc.Forbidden,
            **self.access(self.alt_share['id'], access_type, access_to))

    @decorators.idempotent_id('7a702c74-8d31-49e3-859a-cc8a78d7915e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_access(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'delete_access_rule', expected_status=lib_exc.Forbidden,
            share_id=self.share['id'], rule_id=access['id'])

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'delete_access_rule', expected_status=lib_exc.Forbidden,
            share_id=self.alt_share['id'], rule_id=alt_access['id'])

    @decorators.idempotent_id('a61d7f06-6f0e-4da3-b11d-1c3a0b5bd416')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_access_rule_metadata(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'update_access_metadata', expected_status=lib_exc.Forbidden,
            access_id=access['id'], metadata=self.metadata)

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to)
        self.do_request(
            'update_access_metadata', expected_status=lib_exc.Forbidden,
            access_id=alt_access['id'], metadata=self.metadata)

    @decorators.idempotent_id('5faf0e0b-b246-4392-901d-9e7d628f0d6e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_access_rule_metadata(self):
        access_type, access_to = (
            utils.get_access_rule_data_from_config(self.protocol))
        access = self.allow_access(
            self.share_member_client, self.share['id'],
            access_type=access_type, access_to=access_to,
            metadata=self.metadata)
        self.do_request(
            'delete_access_metadata', expected_status=lib_exc.Forbidden,
            access_id=access['id'], key='key')

        alt_access = self.allow_access(
            self.alt_project_share_v2_client, self.alt_share['id'],
            access_type=access_type, access_to=access_to,
            metadata=self.metadata)
        self.do_request(
            'delete_access_metadata', expected_status=lib_exc.Forbidden,
            access_id=alt_access['id'], key='key')


class TestProjectAdminTestsCEPHFS(TestProjectAdminTestsNFS):
    protocol = 'cephfs'


class TestProjectMemberTestsCEPHFS(TestProjectMemberTestsNFS):
    protocol = 'cephfs'


class TestProjectReaderTestsCEPHFS(TestProjectReaderTestsNFS):
    protocol = 'cephfs'


class TestProjectAdminTestsCIFS(TestProjectAdminTestsNFS):
    protocol = 'cifs'


class TestProjectMemberTestsCIFS(TestProjectMemberTestsNFS):
    protocol = 'cifs'


class TestProjectReaderTestsCIFS(TestProjectReaderTestsNFS):
    protocol = 'cifs'



# Copyright 2016 Huawei
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class ShareSnapshotInstancesTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ShareSnapshotInstancesTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')

        utils.check_skip_if_microversion_not_supported("2.19")

    @classmethod
    def resource_setup(cls):
        super(ShareSnapshotInstancesTest, cls).resource_setup()
        # create share type
        extra_specs = {'snapshot_support': True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)
        snap = cls.create_snapshot_wait_for_active(cls.share["id"])
        cls.snapshot = cls.shares_v2_client.get_snapshot(
            snap['id'])['snapshot']

    @ddt.data(True, False)
    @decorators.idempotent_id('bcb29129-9713-4481-8e74-97682c62f218')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_snapshot_instances_by_snapshot(self, detail):
        """Test that we get only the 1 snapshot instance from snapshot."""
        snapshot_instances = self.shares_v2_client.list_snapshot_instances(
            detail=detail,
            snapshot_id=self.snapshot['id'])['snapshot_instances']

        expected_keys = ['id', 'snapshot_id', 'status']

        if detail:
            extra_detail_keys = ['provider_location', 'share_id',
                                 'share_instance_id', 'created_at',
                                 'updated_at', 'progress']
            expected_keys.extend(extra_detail_keys)

        si_num = len(snapshot_instances)
        self.assertEqual(1, si_num,
                         'Incorrect amount of snapshot instances found; '
                         'expected 1, found %s.' % si_num)

        si = snapshot_instances[0]
        self.assertEqual(self.snapshot['id'], si['snapshot_id'],
                         'Snapshot instance %s has incorrect snapshot id;'
                         ' expected %s, got %s.' % (si['id'],
                                                    self.snapshot['id'],
                                                    si['snapshot_id']))
        if detail:
            self.assertEqual(self.snapshot['share_id'], si['share_id'])

        for key in si:
            self.assertIn(key, expected_keys)
        self.assertEqual(len(expected_keys), len(si))

    @decorators.idempotent_id('1d0bd333-300d-4af2-81eb-176b550f2c36')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_snapshot_instances(self):
        """Test that we get at least the snapshot instance."""
        snapshot_instances = self.shares_v2_client.list_snapshot_instances(
            )['snapshot_instances']

        snapshot_ids = [si['snapshot_id'] for si in snapshot_instances]

        msg = ('Snapshot instance for snapshot %s was not found.' %
               self.snapshot['id'])
        self.assertIn(self.snapshot['id'], snapshot_ids, msg)

    @decorators.idempotent_id('d21bf82d-2b3b-4061-8db5-502b7376a44d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_snapshot_instance(self):
        instances = self.shares_v2_client.list_snapshot_instances(
            snapshot_id=self.snapshot['id'])['snapshot_instances']
        instance_detail = self.shares_v2_client.get_snapshot_instance(
            instance_id=instances[0]['id'])['snapshot_instance']

        expected_keys = (
            'id', 'created_at', 'updated_at', 'progress', 'provider_location',
            'share_id', 'share_instance_id', 'snapshot_id', 'status',
        )

        for key in instance_detail:
            self.assertIn(key, expected_keys)
        self.assertEqual(len(expected_keys), len(instance_detail))
        self.assertEqual(self.snapshot['id'], instance_detail['snapshot_id'])
        self.assertEqual(self.snapshot['share_id'],
                         instance_detail['share_id'])
        self.assertEqual(self.snapshot['provider_location'],
                         instance_detail['provider_location'])

    @decorators.idempotent_id('c7724848-4e17-4d28-801d-2ec77dfcf502')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_snapshot_instance_status_and_delete(self):
        """Test resetting a snapshot instance's status attribute."""
        snapshot = self.create_snapshot_wait_for_active(self.share["id"])

        snapshot_instances = self.shares_v2_client.list_snapshot_instances(
            snapshot_id=snapshot['id'])['snapshot_instances']

        sii = snapshot_instances[0]['id']

        for status in ("error", "available"):
            self.shares_v2_client.reset_snapshot_instance_status(
                sii, status=status)
            waiters.wait_for_resource_status(
                self.shares_v2_client, sii, status,
                resource_name='snapshot_instance')
        self.shares_v2_client.delete_snapshot(snapshot['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            snapshot_id=snapshot['id'])



# Copyright (c) 2017 Hitachi Data Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class SnapshotExportLocationsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SnapshotExportLocationsNegativeTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')
        if not CONF.share.run_mount_snapshot_tests:
            raise cls.skipException('Mountable snapshots tests are disabled.')

        utils.check_skip_if_microversion_not_supported("2.32")

    @classmethod
    def setup_clients(cls):
        super(SnapshotExportLocationsNegativeTest, cls).setup_clients()
        cls.admin_client = cls.admin_shares_v2_client
        cls.different_project_client = cls.alt_shares_v2_client

    @classmethod
    def resource_setup(cls):
        super(SnapshotExportLocationsNegativeTest, cls).resource_setup()
        # create share type
        extra_specs = {
            'snapshot_support': True,
            'mount_snapshot_support': True,
        }
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id,
                                     client=cls.admin_client)
        cls.snapshot = cls.create_snapshot_wait_for_active(
            cls.share['id'], client=cls.admin_client)
        cls.snapshot = cls.admin_client.get_snapshot(
            cls.snapshot['id'])['snapshot']
        cls.snapshot_instances = cls.admin_client.list_snapshot_instances(
            snapshot_id=cls.snapshot['id'])['snapshot_instances']

    @decorators.idempotent_id('53f0f184-7398-4e7a-ac21-fa432570db7f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_inexistent_snapshot_export_location(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.get_snapshot_export_location,
            self.snapshot['id'],
            "fake-inexistent-snapshot-export-location-id",
        )

    @decorators.idempotent_id('43229517-bf93-4be7-9f89-a69034d2f03c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_list_snapshot_export_locations_by_different_project_user(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.different_project_client.list_snapshot_export_locations,
            self.snapshot['id']
        )

    @decorators.idempotent_id('66839514-796a-4ee9-a8ed-7614521d01d5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_snapshot_export_location_by_different_project_user(self):
        export_locations = (
            self.admin_client.list_snapshot_export_locations(
                self.snapshot['id'])['share_snapshot_export_locations'])

        for export_location in export_locations:
            if export_location['is_admin_only']:
                continue
            self.assertRaises(
                lib_exc.NotFound,
                self.different_project_client.get_snapshot_export_location,
                self.snapshot['id'],
                export_location['id'])

    @decorators.idempotent_id('52e0b807-7b29-4795-960a-518bcadc1503')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_inexistent_snapshot_instance_export_location(self):
        for snapshot_instance in self.snapshot_instances:
            self.assertRaises(
                lib_exc.NotFound,
                self.admin_client.get_snapshot_instance_export_location,
                snapshot_instance['id'],
                "fake-inexistent-snapshot-export-location-id",
            )

    @decorators.idempotent_id('3f4e2a0e-1522-47fb-b770-9d7a0651dde2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_snapshot_instance_export_location_by_member(self):
        for snapshot_instance in self.snapshot_instances:
            export_locations = (
                self.admin_client.list_snapshot_instance_export_locations(
                    snapshot_instance['id'])['share_snapshot_export_locations']
            )
            for el in export_locations:
                self.assertRaises(
                    lib_exc.Forbidden,
                    (self.different_project_client.
                     get_snapshot_instance_export_location),
                    snapshot_instance['id'], el['id'],
                )


class SnapshotExportLocationsAPIOnlyNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SnapshotExportLocationsAPIOnlyNegativeTest, cls).skip_checks()
        if not CONF.share.run_snapshot_tests:
            raise cls.skipException('Snapshot tests are disabled.')
        if not CONF.share.run_mount_snapshot_tests:
            raise cls.skipException('Mountable snapshots tests are disabled.')

        utils.check_skip_if_microversion_not_supported('2.32')

    @classmethod
    def setup_clients(cls):
        super(SnapshotExportLocationsAPIOnlyNegativeTest, cls).setup_clients()
        cls.admin_client = cls.admin_shares_v2_client
        # admin_member_client is a regular user in admin's project
        cls.admin_member_client = (
            cls.admin_project_member_client.shares_v2_client)

    @decorators.idempotent_id('37901216-b574-4786-9b1d-9b1ccdf123d2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_export_locations_by_nonexistent_snapshot(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.list_snapshot_export_locations,
            "fake-inexistent-snapshot-id",
        )

    @decorators.idempotent_id('c2aa3770-c061-4b49-83ac-ab29773c2e0c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_export_locations_by_nonexistent_snapshot_instance(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.list_snapshot_instance_export_locations,
            "fake-inexistent-snapshot-instance-id",
        )

    @decorators.idempotent_id('74d5d46d-8161-4e17-acbc-812248d6d694')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_inexistent_snapshot_instance_export_locations_by_member(
            self):
        self.assertRaises(
            lib_exc.Forbidden,
            self.admin_member_client.list_snapshot_instance_export_locations,
            "fake-inexistent-snapshot-instance-id"
        )



# Copyright 2021 Cloudification GmbH
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

FAKE_SHARE_ID = "2d316e9f-39fc-468e-b2d9-634b25ae85f6"
CONF = config.CONF


class SharesSchedulerHintsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SharesSchedulerHintsNegativeTest, cls).skip_checks()
        if not CONF.share.multi_backend:
            raise cls.skipException("Manila multi-backend is disabled.")
        elif len(CONF.share.backend_names) < 2:
            raise cls.skipException("For running multi-backend tests required"
                                    " two names in config. Skipping.")
        elif any(not name for name in CONF.share.backend_names):
            raise cls.skipException("Share backend names can not be empty. "
                                    "Skipping.")
        utils.check_skip_if_microversion_not_supported('2.65')

    @classmethod
    def resource_setup(cls):
        super(SharesSchedulerHintsNegativeTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_a = cls.create_share(share_type_id=cls.share_type_id)

    @decorators.idempotent_id('2228a187-4f03-4195-9e23-fa1a42110fdc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_scheduler_hint_with_invalid_share_id(self):
        scheduler_hint = {"same_host": FAKE_SHARE_ID}
        self.assertRaises(lib_exc.NotFound,
                          self.create_share,
                          share_type_id=self.share_type_id,
                          scheduler_hints=scheduler_hint,
                          cleanup_in_class=False)

    @decorators.idempotent_id('6f0c5561-8a6a-4cfb-bbe7-84ffc39bf78d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_scheduler_hint_with_invalid_hint(self):
        scheduler_hint = {"same_host": "%s" % self.share_a["id"],
                          "different_host": "%s" % self.share_a["id"]}
        self.assertRaises(share_exceptions.ShareBuildErrorException,
                          self.create_share,
                          share_type_id=self.share_type_id,
                          scheduler_hints=scheduler_hint,
                          cleanup_in_class=False)
